# -*- coding: utf-8 -*-
"""
author: zengbin93
email: zeng_bin8888@163.com
create_dt: 2022/10/27 23:23
describe: 用于信号计算函数的各种辅助工具函数
"""
import numpy as np
from deprecated import deprecated
from collections import Counter, OrderedDict
from typing import List, Any, Dict, Union, Tuple, Optional
from czsc.core import Direction
from czsc.core import BI, RawBar, ZS, Signal


def create_single_signal(**kwargs) -> OrderedDict:
    """创建单个信号"""
    from rs_czsc import Signal
    
    s = OrderedDict()
    k1, k2, k3 = kwargs.get("k1", "任意"), kwargs.get("k2", "任意"), kwargs.get("k3", "任意")
    v1, v2, v3 = kwargs.get("v1", "任意"), kwargs.get("v2", "任意"), kwargs.get("v3", "任意")
    score = kwargs.get("score", 0)
    v = Signal(key=f"{k1}_{k2}_{k3}", value=f"{v1}_{v2}_{v3}_{score}")
    # v = Signal(k1=k1, k2=k2, k3=k3, v1=v1, v2=v2, v3=v3, score=kwargs.get("score", 0))
    s[v.key] = v.value
    return s


def is_symmetry_zs(bis: List[BI], th: float = 0.3) -> bool:
    """对称中枢判断：中枢中所有笔的力度序列，标准差小于均值的一定比例

    https://pic2.zhimg.com/80/v2-2f55ef49eda01972462531ebb6de4f19_1440w.jpg

    :param bis: 构成中枢的笔序列
    :param th: 标准差小于均值的比例阈值
    :return:
    """
    if len(bis) % 2 == 0:
        return False

    zs = ZS(bis=bis)
    if zs.zd > zs.zg or max([x.low for x in bis]) > min([x.high for x in bis]):
        return False

    zns = [x.power_price for x in bis]
    if np.std(zns) / np.mean(zns) <= th:
        return True
    else:
        return False


def check_cross_info(fast: Union[List, np.ndarray], slow: Union[List, np.ndarray]):
    """计算 fast 和 slow 的交叉信息

    :param fast: 快线
    :param slow: 慢线
    :return:
    """
    assert len(fast) == len(slow), "快线和慢线的长度不一样"

    if isinstance(fast, list):
        fast = np.array(fast)
    if isinstance(slow, list):
        slow = np.array(slow)

    length = len(fast)
    delta = fast - slow
    cross_info = []
    last_i = -1
    last_v = 0
    temp_fast = []
    temp_slow = []
    for i, v in enumerate(delta):
        last_i += 1
        last_v += abs(v)
        temp_fast.append(fast[i])
        temp_slow.append(slow[i])

        if i >= 2 and delta[i - 1] <= 0 < delta[i]:
            kind = "金叉"
        elif i >= 2 and delta[i - 1] >= 0 > delta[i]:
            kind = "死叉"
        else:
            continue

        cross_info.append(
            {
                "位置": i,
                "类型": kind,
                "快线": fast[i],
                "慢线": slow[i],
                "距离": last_i,
                "距今": length - i,
                "面积": round(last_v, 4),
                "价差": round(v, 4),
                "快线高点": max(temp_fast),
                "快线低点": min(temp_fast),
                "慢线高点": max(temp_slow),
                "慢线低点": min(temp_slow),
            }
        )
        last_i = 0
        last_v = 0
        temp_fast = []
        temp_slow = []

    return cross_info


def check_gap_info(bars: List[RawBar]):
    """检查 bars 中的缺口信息

    :param bars: K线序列，按时间升序
    :return:
    """
    gap_info = []
    if len(bars) < 2:
        return gap_info

    for i in range(1, len(bars)):
        bar1, bar2 = bars[i - 1], bars[i]
        right = bars[i:]

        gap = None
        if bar1.high < bar2.low:
            delta = round(bar2.low / bar1.high - 1, 4)
            cover = "已补" if min(x.low for x in right) < bar1.high else "未补"
            gap = {
                "kind": "向上缺口",
                "cover": cover,
                "sdt": bar1.dt,
                "edt": bar2.dt,
                "high": bar2.low,
                "low": bar1.high,
                "delta": delta,
            }

        if bar1.low > bar2.high:
            delta = round(bar1.low / bar2.high - 1, 4)
            cover = "已补" if max(x.high for x in right) > bar1.low else "未补"
            gap = {
                "kind": "向下缺口",
                "cover": cover,
                "sdt": bar1.dt,
                "edt": bar2.dt,
                "high": bar1.low,
                "low": bar2.high,
                "delta": delta,
            }

        if gap:
            gap_info.append(gap)

    return gap_info


def fast_slow_cross(fast, slow):
    """计算 fast 和 slow 的交叉信息

    :param fast: 快线
    :param slow: 慢线
    :return:
    """
    assert len(fast) == len(slow), "快线和慢线的长度不一样"

    if isinstance(fast, list):
        fast = np.array(fast)
    if isinstance(slow, list):
        slow = np.array(slow)

    length = len(fast)
    delta = fast - slow
    cross_info = []
    last_i = -1
    last_v = 0
    temp_fast = []
    temp_slow = []
    for i, v in enumerate(delta):
        last_i += 1
        last_v += abs(v)
        temp_fast.append(fast[i])
        temp_slow.append(slow[i])

        if i >= 2 and delta[i - 1] <= 0 < delta[i]:
            kind = "金叉"
        elif i >= 2 and delta[i - 1] >= 0 > delta[i]:
            kind = "死叉"
        else:
            continue

        cross_info.append(
            {
                "位置": i,
                "类型": kind,
                "快线": fast[i],
                "慢线": slow[i],
                "距离": last_i,
                "距今": length - i,
                "面积": round(last_v, 4),
                "价差": round(v, 4),
                "快线高点": max(temp_fast),
                "快线低点": min(temp_fast),
                "慢线高点": max(temp_slow),
                "慢线低点": min(temp_slow),
            }
        )
        last_i = 0
        last_v = 0
        temp_fast = []
        temp_slow = []

    return cross_info


def same_dir_counts(seq: Union[List, np.ndarray]):
    """计算 seq 中与最后一个数字同向的数字数量

    :param seq: 数字序列
    :return:

    example
    ----------
    >>>print(same_dir_counts([-1, -1, -2, -3, 0, 1, 2, 3, -1, -2, 1, 1, 2, 3]))
    >>>print(same_dir_counts([-1, -1, -2, -3, 0, 1, 2, 3]))
    """
    s = seq[-1]
    c = 0
    for num in seq[::-1]:
        if (num > 0 and s > 0) or (num < 0 and s < 0):
            c += 1
        else:
            break
    return c


def count_last_same(seq: Union[List, np.ndarray, Tuple]):
    """统计与seq列表最后一个元素相似的连续元素数量

    :param seq: 数字序列
    :return:
    """
    s = seq[-1]
    c = 0
    for _s in seq[::-1]:
        if _s == s:
            c += 1
        else:
            break
    return c


def get_sub_elements(elements: List[Any], di: int = 1, n: int = 10) -> List[Any]:
    """获取截止到倒数第 di 个元素的前 n 个元素

    :param elements: 全部元素列表
    :param di: 指定结束元素为倒数第 di 个
    :param n: 指定需要的元素个数
    :return: 部分元素列表

    >>>x = [1, 2, 3, 4, 5, 6, 7, 8, 9]
    >>>y1 = get_sub_elements(x, di=1, n=3)
    >>>y2 = get_sub_elements(x, di=2, n=3)
    """
    assert di >= 1
    if di == 1:
        se = elements[-n:]
    else:
        se = elements[-n - di + 1 : -di + 1]
    return se


def is_bis_down(bis: List[BI]):
    """判断 bis 中的连续笔是否是向下的"""
    if not bis or len(bis) < 3 or len(bis) % 2 == 0:
        return False

    assert bis[1].fx_b.dt > bis[0].fx_b.dt, "时间由远到近"

    if (
        bis[-1].direction == Direction.Down
        and bis[0].high == max([x.high for x in bis])
        and bis[-1].low == min([x.low for x in bis])
    ):
        return True
    else:
        return False


def is_bis_up(bis: List[BI]):
    """判断 bis 中的连续笔是否是向上的"""
    if not bis or len(bis) < 3 and len(bis) % 2 == 0:
        return False

    assert bis[1].fx_b.dt > bis[0].fx_b.dt, "时间由远到近"

    if (
        bis[-1].direction == Direction.Up
        and bis[-1].high == max([x.high for x in bis])
        and bis[0].low == min([x.low for x in bis])
    ):
        return True
    else:
        return False


def get_zs_seq(bis: List[BI]) -> List[ZS]:
    """获取连续笔中的中枢序列

    :param bis: 连续笔对象列表
    :return: 中枢序列
    """
    zs_list = []
    if not bis:
        return []

    for bi in bis:
        if not zs_list:
            zs_list.append(ZS(bis=[bi]))
            continue

        zs = zs_list[-1]
        if not zs.bis:
            zs.bis.append(bi)
            zs_list[-1] = zs
        else:
            if (bi.direction == Direction.Up and bi.high < zs.zd) or (
                bi.direction == Direction.Down and bi.low > zs.zg
            ):
                zs_list.append(ZS(bis=[bi]))
            else:
                zs.bis.append(bi)
                zs_list[-1] = zs
    return zs_list


def cross_zero_axis(n1: Union[List, np.ndarray], n2: Union[List, np.ndarray]) -> int:
    """判断两个数列的零轴交叉点

    :param n1: 数列1
    :param n2: 数列2
    :return: 交叉点所在的索引位置
    """
    assert len(n1) == len(n2), "输入两个数列长度不等"
    axis_0 = np.zeros(len(n1))

    n1 = np.flip(n1)
    n2 = np.flip(n2)

    x1 = np.where(n1[0] * n1 < axis_0, True, False)
    x2 = np.where(n2[0] * n2 < axis_0, True, False)

    num1 = np.argmax(x1[:-1] != x1[1:]) + 2 if np.any(x1) else 0
    num2 = np.argmax(x2[:-1] != x2[1:]) + 2 if np.any(x2) else 0
    return int(max(num1, num2))


def cal_cross_num(cross: List, distance: int = 1) -> tuple:
    """使用 distance 过滤掉fast_slow_cross函数返回值cross列表中
    不符合要求的交叉点，返回处理后的金叉和死叉数值

    :param cross: fast_slow_cross函数返回值
    :param distance: 金叉和死叉之间的最小距离
    :return: jc金叉值 ，SC死叉值
    """
    if len(cross) == 0:
        return 0, 0
    elif len(cross) == 1:
        cross_ = cross
    elif len(cross) == 2:
        if cross[-1]["距离"] < distance:
            cross_ = []
        else:
            cross_ = cross
    else:
        if cross[-1]["距离"] < distance:
            last_cross = cross[-1]
            del cross[-2]
            re_cross = [i for i in cross if i["距离"] >= distance]
            re_cross.append(last_cross)
        else:
            re_cross = [i for i in cross if i["距离"] >= distance]
        cross_ = []
        for i in range(0, len(re_cross)):
            if len(cross_) >= 1 and re_cross[i]["类型"] == re_cross[i - 1]["类型"]:
                # 不将上一个元素加入cross_
                del cross_[-1]
                cross_.append(re_cross[i])
            else:
                cross_.append(re_cross[i])

    jc = len([x for x in cross_ if x["类型"] == "金叉"])
    sc = len([x for x in cross_ if x["类型"] == "死叉"])

    return jc, sc


def down_cross_count(x1: Union[List, np.ndarray], x2: Union[List, np.ndarray]) -> int:
    """输入两个序列，计算 x1 下穿 x2 的次数

    :param x1: list
    :param x2: list
    :return: int
    """
    x = np.array(x1) < np.array(x2)
    num = 0
    for i in range(len(x) - 1):
        b1, b2 = x[i], x[i + 1]
        if b2 and b1 != b2:
            num += 1
    return num



# -*- coding: utf-8 -*-
"""
author: zengbin93
email: zeng_bin8888@163.com
create_dt: 2024/12/30 15:19
describe: 基于 clickhouse 的策略持仓权重管理，cwc 为 clickhouse weights client 的缩写

时区标记：https://clockhub.app/zh-CN/timezone

推荐在环境变量中设置 clickhouse 的连接信息，如下：

- CLICKHOUSE_HOST: 服务器地址，如 127.0.0.1
- CLICKHOUSE_PORT: 服务器端口，如 9000
- CLICKHOUSE_USER: 用户名, 如 default
- CLICKHOUSE_PASS: 密码, 如果没有密码，可以设置为空字符串

额外可选配置：

- CLICKHOUSE_CONNECT_TIMEOUT: 建立连接的超时时间（秒），默认为 10
- CLICKHOUSE_SEND_RECEIVE_TIMEOUT: 发送/接收（读写）的超时时间（秒），默认为 60
"""
# pip install clickhouse_connect -i https://pypi.tuna.tsinghua.edu.cn/simple
import os
from typing import Optional

import clickhouse_connect as ch
import loguru
import pandas as pd
from clickhouse_connect.driver.client import Client
from zoneinfo import ZoneInfo


def _ensure_timestamp(value, tz=ZoneInfo("Asia/Shanghai")):
    """将任意时间对象转换为带时区的 Timestamp。

    :param value: 时间值
    :param tz: 时区，默认 Asia/Shanghai
    :return: 带时区的 Timestamp
    """
    if value is None:
        return pd.NaT
    if isinstance(value, str) and not value.strip():
        return pd.NaT

    ts = pd.to_datetime(value, errors="coerce")
    if pd.isna(ts):
        return pd.NaT

    if ts.tzinfo is None:
        return ts.tz_localize(tz)
    return ts.tz_convert(tz)


def _ensure_series_tz(series: pd.Series, tz=ZoneInfo("Asia/Shanghai")) -> pd.Series:
    """确保 Series 中的时间字段带有指定时区。

    :param series: 时间序列
    :param tz: 时区，默认 Asia/Shanghai
    :return: 带时区的 Series
    """
    from pandas.api.types import is_datetime64tz_dtype

    ser = pd.to_datetime(series, errors="coerce")
    if is_datetime64tz_dtype(ser):
        return ser.dt.tz_convert(tz)
    return ser.dt.tz_localize(tz)


def _format_for_db(value, tz=ZoneInfo("Asia/Shanghai")) -> Optional[str]:
    """将带时区的时间对象格式化为 ClickHouse 兼容的字符串。

    :param value: 时间值
    :param tz: 时区，默认 Asia/Shanghai
    :return: 格式化的时间字符串
    """
    ts = _ensure_timestamp(value, tz=tz)
    if pd.isna(ts):
        return None
    return ts.astimezone(tz).replace(tzinfo=None).strftime("%Y-%m-%d %H:%M:%S")


def _localize_dataframe_columns(df: pd.DataFrame, columns, tz=ZoneInfo("Asia/Shanghai")) -> pd.DataFrame:
    """将 DataFrame 中指定列本地化为指定时区。

    :param df: DataFrame
    :param columns: 需要转换的列名列表
    :param tz: 时区，默认 Asia/Shanghai
    :return: 转换后的 DataFrame
    """
    for col in columns:
        if col in df.columns:
            df[col] = _ensure_series_tz(df[col], tz=tz)
    return df


def __db_from_env():
    host = os.getenv("CLICKHOUSE_HOST")
    port_str = os.getenv("CLICKHOUSE_PORT")
    port = int(port_str) if port_str else None
    user = os.getenv("CLICKHOUSE_USER")
    password = os.getenv("CLICKHOUSE_PASS")
    # 建立连接的超时时间（秒）
    connect_timeout = int(os.getenv("CLICKHOUSE_CONNECT_TIMEOUT", 10))
    # 发送/接收（读写）的超时时间（秒）
    send_receive_timeout = int(os.getenv("CLICKHOUSE_SEND_RECEIVE_TIMEOUT", 60))

    if not (host and port and user and password):
        raise ValueError(
            """
        请设置环境变量：
        
        # 必须
        - CLICKHOUSE_HOST: 服务器地址，如 127.0.0.1
        - CLICKHOUSE_PORT: 服务器端口，如 9000
        - CLICKHOUSE_USER: 用户名, 如 default
        - CLICKHOUSE_PASS: 密码, 如果没有密码，可以设置为空字符串
        
        # 可选
        - CLICKHOUSE_CONNECT_TIMEOUT: 建立连接的超时时间（秒），默认为 10
        - CLICKHOUSE_SEND_RECEIVE_TIMEOUT: 发送/接收（读写）的超时时间（秒），默认为 60
        - CZSC_TIMEZONE: 时区名称，例如 Asia/Shanghai，默认使用 Asia/Shanghai；若系统不支持该时区，则回退到 UTC

        """
        )

    db = ch.get_client(host=host, port=port, user=user, password=password,
                       connect_timeout=connect_timeout, 
                       send_receive_timeout=send_receive_timeout)
    return db


def init_latest_weights_view(db: Optional[Client] = None, database="czsc_strategy", **kwargs):
    """
    策略类型有 cs 和 ts，这两种策略对应的写入逻辑有所区别，需要单独创建最新持仓的视图，然后再合并。
    """
    db = db or __db_from_env()
    logger = kwargs.get('logger', loguru.logger)
    
    # 创建截面策略的最新持仓视图
    cs_view_sql = f"""
    CREATE VIEW IF NOT EXISTS {database}.cs_latest_weights AS
    WITH latest_dates AS (
        SELECT 
            strategy,
            MAX(dt) AS latest_dt
        FROM {database}.weights
        GROUP BY strategy
    )
    SELECT 
        w.dt as dt,
        w.symbol as symbol,
        w.weight as weight,
        w.strategy as strategy,
        w.update_time as update_time
    FROM {database}.weights w
    JOIN latest_dates ld ON w.strategy = ld.strategy AND w.dt = ld.latest_dt
    JOIN {database}.metas m ON w.strategy = m.strategy
    WHERE m.weight_type = 'cs'
    """
    db.command(cs_view_sql)
    logger.info("cs_latest_weights 视图初始化完成")

    # 创建时序策略的最新持仓视图
    ts_view_sql = f"""
    CREATE VIEW IF NOT EXISTS {database}.ts_latest_weights AS
    WITH latest_records AS (
        SELECT 
            strategy,
            symbol,
            MAX(dt) AS latest_dt
        FROM {database}.weights
        GROUP BY strategy, symbol
    )
    SELECT 
        w.dt as dt,
        w.symbol as symbol,
        w.weight as weight,
        w.strategy as strategy,
        w.update_time as update_time
    FROM {database}.weights w
    JOIN latest_records lr ON w.strategy = lr.strategy 
                          AND w.symbol = lr.symbol 
                          AND w.dt = lr.latest_dt
    JOIN {database}.metas m ON w.strategy = m.strategy
    WHERE m.weight_type = 'ts'
    """
    db.command(ts_view_sql)
    logger.info("ts_latest_weights 视图初始化完成")

    # 创建合并的最新持仓视图
    latest_view_sql = f"""
    CREATE VIEW IF NOT EXISTS {database}.latest_weights AS
    SELECT * FROM {database}.ts_latest_weights
    UNION ALL
    SELECT * FROM {database}.cs_latest_weights
    """
    db.command(latest_view_sql)
    logger.info("latest_weights 视图初始化完成")
    
    logger.info("所有最新持仓权重视图初始化完成")

def init_tables(db: Optional[Client] = None, database="czsc_strategy", **kwargs):
    """
    创建数据库表

    :param db: clickhouse_connect.driver.Client, 数据库连接
    :param database: str, 数据库名称
    :param kwargs: dict, 数据表名和建表语句
    :return: None
    """
    logger = kwargs.get('logger', loguru.logger)
    
    db = db or __db_from_env()

    # 创建数据库
    db.command(f"CREATE DATABASE IF NOT EXISTS {database}")

    metas_table = f"""
    CREATE TABLE IF NOT EXISTS {database}.metas (
        strategy String NOT NULL,                              -- 策略名（唯一且不能为空）
        base_freq String,                                      -- 周期
        description String,                                    -- 描述
        author String,                                         -- 作者
        outsample_sdt DateTime('Asia/Shanghai'),               -- 样本外起始时间
        create_time DateTime('Asia/Shanghai'),                 -- 策略入库时间
        update_time DateTime('Asia/Shanghai'),                 -- 策略更新时间
        heartbeat_time DateTime('Asia/Shanghai'),              -- 最后一次心跳时间
        weight_type String,                                    -- 策略上传的权重类型，ts 或 cs
        status String DEFAULT '实盘',                           -- 策略状态：实盘、废弃
        memo String                                            -- 策略备忘信息
    )
    ENGINE = ReplacingMergeTree()
    ORDER BY strategy;
    """

    weights_table = f"""
    CREATE TABLE IF NOT EXISTS {database}.weights (
        dt DateTime('Asia/Shanghai'),            -- 持仓权重时间
        symbol String,                           -- 符号（例如，股票代码或其他标识符）
        weight Float64,                          -- 策略持仓权重值
        strategy String,                         -- 策略名称
        update_time DateTime('Asia/Shanghai')    -- 持仓权重更新时间
    )
    ENGINE = ReplacingMergeTree()
    ORDER BY (strategy, dt, symbol);
    """

    returns_table = f"""
    CREATE TABLE IF NOT EXISTS {database}.returns (
        dt DateTime('Asia/Shanghai'),            -- 时间
        symbol String,                           -- 符号（例如，股票代码或其他标识符）
        returns Float64,                         -- 策略收益，从上一个 dt 到当前 dt 的收益
        strategy String,                         -- 策略名称
        update_time DateTime('Asia/Shanghai')    -- 更新时间
    )
    ENGINE = ReplacingMergeTree()
    ORDER BY (strategy, dt, symbol);
    """

    db.command(metas_table)
    logger.info("metas 表创建成功！")
    db.command(weights_table)
    logger.info("weights 表创建成功！")
    db.command(returns_table)
    logger.info("returns 表创建成功！")


def initialize(db: Optional[Client] = None, database="czsc_strategy", **kwargs):
    """初始化数据库，包括创建数据表和最新持仓视图

    :param db: clickhouse_connect.driver.Client, 数据库连接
    :param database: str, 数据库名称
    :param kwargs: dict, 其他参数
    :return: None
    """
    db = db or __db_from_env()
    init_tables(db=db, database=database, **kwargs)
    init_latest_weights_view(db=db, database=database, **kwargs)


def get_meta(
    strategy,
    db: Optional[Client] = None,
    database="czsc_strategy",
    logger=loguru.logger,
    tz=ZoneInfo("Asia/Shanghai")
) -> dict:
    """获取策略元数据

    :param strategy: str, 策略名称
    :param db: clickhouse_connect.driver.Client, 数据库连接
    :param database: str, 数据库名称
    :param logger: loguru.logger, 日志记录器
    :param tz: 时区，默认 Asia/Shanghai
    :return: dict
    """
    db = db or __db_from_env()
    df = db.query_df(f"SELECT * FROM {database}.metas final WHERE strategy = '{strategy}'")

    if df.empty:
        logger.warning(f"策略 {strategy} 不存在元数据")
        return {}

    assert len(df) == 1, f"策略 {strategy} 存在多条元数据，请检查"
    df = _localize_dataframe_columns(df, ["outsample_sdt", "create_time", "update_time", "heartbeat_time"], tz=tz)
    return df.iloc[0].to_dict()


def get_all_metas(
    db: Optional[Client] = None,
    database="czsc_strategy",
    tz=ZoneInfo("Asia/Shanghai")
) -> pd.DataFrame:
    """获取所有策略元数据

    :param db: clickhouse_connect.driver.Client, 数据库连接
    :param database: str, 数据库名称
    :param tz: 时区，默认 Asia/Shanghai
    :return: pd.DataFrame
    """
    db = db or __db_from_env()
    df = db.query_df(f"SELECT * FROM {database}.metas final")
    if not df.empty:
        df = _localize_dataframe_columns(df, ["outsample_sdt", "create_time", "update_time", "heartbeat_time"], tz=tz)
    return df


def set_meta(
    strategy,
    base_freq,
    description,
    author,
    outsample_sdt,
    weight_type="ts",
    status="实盘",
    memo="",
    logger=loguru.logger,
    overwrite=False,
    database="czsc_strategy",
    db: Optional[Client] = None,
    tz=ZoneInfo("Asia/Shanghai"),
):
    """设置策略元数据

    :param strategy: str, 策略名
    :param base_freq: str, 周期
    :param description: str, 描述
    :param author: str, 作者
    :param outsample_sdt: str, 样本外起始时间
    :param weight_type: str, 权重类型，ts 或 cs
    :param status: str, 策略状态，实盘 或 废弃
    :param memo: str, 备注
    :param logger: loguru.logger, 日志记录器
    :param overwrite: bool, 是否覆盖已有元数据
    :param database: str, 数据库名称
    :param db: clickhouse_connect.driver.Client, 数据库连接
    :param tz: 时区，默认 Asia/Shanghai
    :return: None
    """
    db = db or __db_from_env()

    outsample_sdt = _ensure_timestamp(outsample_sdt, tz=tz)
    current_time = pd.Timestamp.now(tz=tz)
    meta = get_meta(db=db, strategy=strategy, database=database, tz=tz)

    if not overwrite and meta:
        logger.warning(f"策略 {strategy} 已存在元数据，如需更新请设置 overwrite=True")
        return

    create_time = current_time if not meta else _ensure_timestamp(meta.get("create_time"), tz=tz)

    # 构建DataFrame用于插入
    df = pd.DataFrame(
        [
            {
                "strategy": strategy,
                "base_freq": base_freq,
                "description": description,
                "author": author,
                "outsample_sdt": outsample_sdt,
                "create_time": create_time,
                "update_time": current_time,
                "heartbeat_time": current_time,
                "weight_type": weight_type,
                "status": status,
                "memo": memo,
            }
        ]
    )
    res = db.insert_df(f"{database}.metas", df)
    logger.info(f"{strategy} set_metadata: {res.summary}")


def __send_heartbeat(db: Client, strategy, logger=loguru.logger, database="czsc_strategy", tz=ZoneInfo("Asia/Shanghai")):
    """发送心跳

    :param db: clickhouse_connect.driver.Client, 数据库连接
    :param strategy: str, 策略名称
    :param logger: loguru.logger, 日志记录器
    :param database: str, 数据库名称
    :param tz: 时区，默认 Asia/Shanghai
    :return: None
    """
    try:
        meta = get_meta(db=db, strategy=strategy, database=database, logger=logger, tz=tz)
        if not meta:
            logger.warning(f"策略 {strategy} 不存在元数据，无法发送心跳")
            return

        current_time = _format_for_db(pd.Timestamp.now(tz=tz), tz=tz)
        db.command(
            f"ALTER TABLE {database}.metas UPDATE heartbeat_time = '{current_time}' WHERE strategy = '{strategy}'"
        )
        logger.info(f"策略 {strategy} 发送心跳成功")
    except Exception as e:
        logger.error(f"发送心跳失败: {e}")
        raise


def get_strategy_weights(
    strategy, db: Optional[Client] = None, sdt=None, edt=None, symbols=None, database="czsc_strategy", tz=ZoneInfo("Asia/Shanghai")
):
    """获取策略持仓权重

    :param db: clickhouse_connect.driver.Client, 数据库连接
    :param strategy: str, 策略名称
    :param sdt: str, 开始时间
    :param edt: str, 结束时间
    :param symbols: list, 符号列表
    :param database: str, 数据库名称
    :return: pd.DataFrame
    """
    db = db or __db_from_env()

    query = f"""
    SELECT * FROM {database}.weights final WHERE strategy = '{strategy}'
    """
    if sdt:
        sdt_str = _format_for_db(sdt)
        if sdt_str:
            query += f" AND dt >= '{sdt_str}'"
    if edt:
        edt_str = _format_for_db(edt)
        if edt_str:
            query += f" AND dt <= '{edt_str}'"
    if symbols:
        if isinstance(symbols, str):
            symbols = [symbols]
        symbol_str = ", ".join([f"'{s}'" for s in symbols])
        query += f""" AND symbol IN ({symbol_str})"""

    df = db.query_df(query)
    if not df.empty:
        df = _localize_dataframe_columns(df, ["dt", "update_time"], tz=tz)
        df = df.sort_values(["dt", "symbol"]).reset_index(drop=True)
    return df


def get_latest_weights(db: Optional[Client] = None, strategy=None, database="czsc_strategy", tz=ZoneInfo("Asia/Shanghai")) -> pd.DataFrame:
    """获取策略最新持仓权重时间

    :param db: clickhouse_connect.driver.Client, 数据库连接
    :param strategy: str, 策略名称, 默认 None
    :param database: str, 数据库名称
    :return: pd.DataFrame
    """
    db = db or __db_from_env()

    query = f"SELECT * FROM {database}.latest_weights final"
    if strategy:
        query += f" WHERE strategy = '{strategy}'"

    df = db.query_df(query)
    df = df.rename(columns={"latest_dt": "dt", "latest_weight": "weight", "latest_update_time": "update_time"})
    if not df.empty:
        df = _localize_dataframe_columns(df, ["dt", "update_time"], tz=tz)
        df = df.sort_values(["strategy", "dt", "symbol"]).reset_index(drop=True)
    return df


def publish_weights(
    strategy: str,
    df: pd.DataFrame,
    batch_size=100000,
    logger=loguru.logger,
    db: Optional[Client] = None,
    database="czsc_strategy",
    tz=ZoneInfo("Asia/Shanghai"),
):
    """发布策略持仓权重

    :param df: pd.DataFrame, 待发布的持仓权重数据
    :param db: clickhouse_connect.driver.Client, 数据库连接
    :param strategy: str, 策略名称
    :param batch_size: int, 批量发布的大小, 默认 100000
    :param logger: loguru.logger, 日志记录器
    :param database: str, 数据库名称
    :return: None
    """
    db = db or __db_from_env()

    __send_heartbeat(db, strategy, database=database, logger=logger, tz=tz)
    df = df[["dt", "symbol", "weight"]].copy()
    df["strategy"] = strategy
    df["dt"] = _ensure_series_tz(df["dt"], tz=tz)

    dfl = get_latest_weights(db, strategy, tz=tz, database=database)

    if not dfl.empty:
        dfl = _localize_dataframe_columns(dfl, ["dt"], tz=tz)
        symbol_dt = dfl.set_index("symbol")["dt"].to_dict()
        logger.info(f"策略 {strategy} 最新时间：{dfl['dt'].max()}")

        rows = []
        for symbol, dfg in df.groupby("symbol"):
            if symbol in symbol_dt:
                dfg = dfg[dfg["dt"] > symbol_dt[symbol]].copy().reset_index(drop=True)
            rows.append(dfg)
        
        if rows:
            df = pd.concat(rows, ignore_index=True)

        logger.info(f"策略 {strategy} 共 {len(df)} 条新信号")

    df = df.sort_values(["dt", "symbol"]).reset_index(drop=True)
    df["update_time"] = pd.Timestamp.now(tz=tz)
    df = df[["strategy", "symbol", "dt", "weight", "update_time"]].copy()
    df = df.drop_duplicates(["symbol", "dt", "strategy"], keep="last").reset_index(drop=True)
    df["weight"] = df["weight"].astype(float)

    logger.info(f"准备发布 {len(df)} 条策略信号")

    # 批量写入
    for i in range(0, len(df), batch_size):
        batch_df = df.iloc[i : i + batch_size]
        res = db.insert_df(f"{database}.weights", batch_df)
        __send_heartbeat(db, strategy, tz=tz, database=database, logger=logger)

        if res:
            logger.info(f"完成批次 {i//batch_size + 1}, 发布 {len(batch_df)} 条信号")
        else:
            logger.error(f"批次 {i//batch_size + 1} 发布失败: {res}")
            return

    logger.info(f"完成所有信号发布, 共 {len(df)} 条")
    __send_heartbeat(db, strategy, tz=tz, database=database, logger=logger)


def publish_returns(
    strategy: str,
    df: pd.DataFrame,
    batch_size=100000,
    logger=loguru.logger,
    database="czsc_strategy",
    db: Optional[Client] = None,
    tz=ZoneInfo("Asia/Shanghai"),
):
    """发布策略日收益

    :param df: pd.DataFrame, 待发布的日收益数据, 必须包含 dt, symbol, returns 三列
    :param db: clickhouse_connect.driver.Client, 数据库连接
    :param strategy: str, 策略名称
    :param batch_size: int, 批量发布的大小, 默认 100000
    :param logger: loguru.logger, 日志记录器
    :return: None
    """
    db = db or __db_from_env()

    df = df[["dt", "symbol", "returns"]].copy()
    df["strategy"] = strategy
    df["dt"] = _ensure_series_tz(df["dt"], tz=tz)

    # 查询 czsc_strategy.returns 表中，每个品种最新的时间
    dfl = db.query_df(
        f"SELECT symbol, max(dt) as dt FROM {database}.returns final WHERE strategy = '{strategy}' GROUP BY symbol"
    )

    if not dfl.empty:
        dfl["dt"] = _ensure_series_tz(dfl["dt"], tz=tz)
        symbol_dt = dfl.set_index("symbol")["dt"].to_dict()
        logger.info(f"策略 {strategy} 最新时间：{dfl['dt'].max()}")

        rows = []
        for symbol, dfg in df.groupby("symbol"):
            if symbol in symbol_dt:
                # 允许覆盖同一天的数据
                dfg = dfg[dfg["dt"] >= symbol_dt[symbol]].copy()
            rows.append(dfg)
        if rows:
            df = pd.concat(rows, ignore_index=True)

        logger.info(f"策略 {strategy} 共 {len(df)} 条新日收益")

    df = df.sort_values(["dt", "symbol"]).reset_index(drop=True)
    df["update_time"] = pd.Timestamp.now(tz=tz)
    df = df[["strategy", "symbol", "dt", "returns", "update_time"]].copy()
    df = df.drop_duplicates(["symbol", "dt", "strategy"], keep="last").reset_index(drop=True)
    df["returns"] = df["returns"].astype(float)

    logger.info(f"准备发布 {len(df)} 条策略日收益")

    # 批量写入
    for i in range(0, len(df), batch_size):
        batch_df = df.iloc[i : i + batch_size]
        res = db.insert_df(f"{database}.returns", batch_df)

        if res:
            logger.info(f"完成批次 {i//batch_size + 1}, 发布 {len(batch_df)} 条日收益")
        else:
            logger.error(f"批次 {i//batch_size + 1} 发布失败")
            return

    logger.info(f"完成所有日收益发布, 共 {len(df)} 条")


def get_strategy_returns(
    strategy, db: Optional[Client] = None, sdt=None, edt=None, symbols=None, database="czsc_strategy", tz=ZoneInfo("Asia/Shanghai")
):
    """获取策略日收益

    :param db: clickhouse_connect.driver.Client, 数据库连接
    :param strategy: str, 策略名称
    :param sdt: str, 开始时间
    :param edt: str, 结束时间
    :param symbols: list, 符号列表
    :param database: str, 数据库名称
    :return: pd.DataFrame
    """
    db = db or __db_from_env()

    query = f"""
    SELECT * FROM {database}.returns final WHERE strategy = '{strategy}'
    """
    if sdt:
        sdt_ts = _ensure_timestamp(sdt).tz_convert(tz)
        sdt_ts = sdt_ts.replace(hour=0, minute=0, second=0, microsecond=0)
        sdt_str = _format_for_db(sdt_ts)
        query += f" AND dt >= '{sdt_str}'"
    if edt:
        edt_ts = _ensure_timestamp(edt).tz_convert(tz)
        edt_ts = edt_ts.replace(hour=23, minute=59, second=59, microsecond=0)
        edt_str = _format_for_db(edt_ts)
        query += f" AND dt <= '{edt_str}'"
    if symbols:
        if isinstance(symbols, str):
            symbols = [symbols]
        symbol_str = ", ".join([f"'{s}'" for s in symbols])
        query += f""" AND symbol IN ({symbol_str})"""

    df = db.query_df(query)
    if not df.empty:
        df = _localize_dataframe_columns(df, ["dt", "update_time"], tz=tz)
        df = df.sort_values(["dt", "symbol"]).reset_index(drop=True)
    return df


def update_strategy_status(
    strategy, status, db: Optional[Client] = None, logger=loguru.logger, database="czsc_strategy", tz=ZoneInfo("Asia/Shanghai")
):
    """更新策略状态

    :param strategy: str, 策略名称
    :param status: str, 策略状态，实盘 或 废弃
    :param db: clickhouse_connect.driver.Client, 数据库连接
    :param logger: loguru.logger, 日志记录器
    :param database: str, 数据库名称
    :return: None
    """
    db = db or __db_from_env()

    # 验证状态值
    valid_statuses = ["实盘", "废弃"]
    if status not in valid_statuses:
        raise ValueError(f"无效的策略状态: {status}，有效状态为: {valid_statuses}")

    # 检查策略是否存在
    meta = get_meta(db=db, strategy=strategy, database=database, logger=logger, tz=tz)
    if not meta:
        logger.warning(f"策略 {strategy} 不存在，无法更新状态")
        return

    current_time = _format_for_db(pd.Timestamp.now(tz=tz), tz=tz)

    # 更新策略状态和更新时间
    query = f"""
    ALTER TABLE {database}.metas 
    UPDATE status = %s, update_time = %s 
    WHERE strategy = %s
    """

    db.command(query, parameters=(status, current_time, strategy))
    logger.info(f"策略 {strategy} 状态已更新为: {status}")


def get_strategies_by_status(status=None, db: Optional[Client] = None, database="czsc_strategy", tz=ZoneInfo("Asia/Shanghai")) -> pd.DataFrame:
    """根据状态获取策略列表

    :param status: str, 策略状态，实盘 或 废弃，None 表示获取所有状态
    :param db: clickhouse_connect.driver.Client, 数据库连接
    :param database: str, 数据库名称
    :return: pd.DataFrame
    """
    db = db or __db_from_env()

    query = f"SELECT * FROM {database}.metas final"
    if status:
        query += f" WHERE status = '{status}'"

    df = db.query_df(query)
    if not df.empty:
        df = _localize_dataframe_columns(df, ["outsample_sdt", "create_time", "update_time", "heartbeat_time"], tz=tz)
    return df


def clear_strategy(
    strategy, db: Optional[Client] = None, logger=loguru.logger, human_confirm=True, database="czsc_strategy", tz=ZoneInfo("Asia/Shanghai")
):
    """清空策略

    :param db: clickhouse_connect.driver.Client, 数据库连接
    :param strategy: str, 策略名称
    :param logger: loguru.logger, 日志记录器
    :param human_confirm: bool, 是否需要人工确认，默认 True
    :param database: str, 数据库名称
    :return: None
    """
    db = db or __db_from_env()

    # 删除前，先查询跟这个策略相关的数据情况
    meta = get_meta(db=db, strategy=strategy, database=database, logger=logger, tz=tz)
    if not meta:
        logger.warning(f"策略 {strategy} 不存在，无需清空")
        return

    # 统计各个表中的数据量
    try:
        # 统计权重数据量
        weights_count = db.query_df(
            f"SELECT count(*) as count FROM {database}.weights final WHERE strategy = '{strategy}'"
        ).iloc[0]["count"]

        # 统计收益数据量
        returns_count = db.query_df(
            f"SELECT count(*) as count FROM {database}.returns final WHERE strategy = '{strategy}'"
        ).iloc[0]["count"]

        # 获取权重数据的时间范围
        weights_time_range = db.query_df(
            f"""
            SELECT min(dt) as min_dt, max(dt) as max_dt 
            FROM {database}.weights final 
            WHERE strategy = '{strategy}'
        """
        )

        # 获取收益数据的时间范围
        returns_time_range = db.query_df(
            f"""
            SELECT min(dt) as min_dt, max(dt) as max_dt 
            FROM {database}.returns final 
            WHERE strategy = '{strategy}'
        """
        )

        # 输出数据概况
        logger.info(f"策略 {strategy} 数据概况:")
        logger.info(f"  - 策略状态: {meta.get('status', '未知')}")
        logger.info(f"  - 创建时间: {meta.get('create_time', '未知')}")
        logger.info(f"  - 最后更新: {meta.get('update_time', '未知')}")
        logger.info(f"  - 权重数据: {weights_count:,} 条")

        if not weights_time_range.empty:
            weights_time_range = _localize_dataframe_columns(weights_time_range, ["min_dt", "max_dt"], tz=tz)
        if not weights_time_range.empty and weights_time_range.iloc[0]["min_dt"] is not None:
            min_dt = weights_time_range.iloc[0]["min_dt"]
            max_dt = weights_time_range.iloc[0]["max_dt"]
            logger.info(f"    时间范围: {min_dt} 至 {max_dt}")

        logger.info(f"  - 收益数据: {returns_count:,} 条")

        if not returns_time_range.empty:
            returns_time_range = _localize_dataframe_columns(returns_time_range, ["min_dt", "max_dt"], tz=tz)
        if not returns_time_range.empty and returns_time_range.iloc[0]["min_dt"] is not None:
            min_dt = returns_time_range.iloc[0]["min_dt"]
            max_dt = returns_time_range.iloc[0]["max_dt"]
            logger.info(f"    时间范围: {min_dt} 至 {max_dt}")

        total_records = weights_count + returns_count + 1  # +1 for meta record
        logger.info(f"  - 总计将删除: {total_records:,} 条记录")

    except Exception as e:
        logger.error(f"查询策略 {strategy} 数据概况失败: {e}")
        logger.info("将继续执行删除操作...")

    if human_confirm:
        logger.info("\n" + "=" * 60)
        logger.info(f"⚠️  警告：即将删除策略 {strategy} 的所有数据")
        logger.info("=" * 60)
        confirm = input("请仔细确认上述信息，确认删除请输入 'DELETE' (大小写敏感): ")
        if confirm != "DELETE":
            logger.warning(f"取消清空策略 {strategy} 的所有数据")
            return
        logger.info("开始执行删除操作...")

    query = f"""
    DELETE FROM {database}.metas WHERE strategy = '{strategy}'
    """
    _ = db.command(query)
    logger.info(f"清空策略 {strategy} 元数据成功")

    query = f"""
    DELETE FROM {database}.weights WHERE strategy = '{strategy}'
    """
    _ = db.command(query)
    logger.info(f"清空策略 {strategy} 持仓权重成功")

    query = f"""
    DELETE FROM {database}.returns WHERE strategy = '{strategy}'
    """
    _ = db.command(query)
    logger.info(f"清空策略 {strategy} 日收益成功")
    logger.warning(f"策略 {strategy} 清空完成")



""" 
权重回测 HTML 报告生成器

使用 Python f-string + plotly 绘图实现 WeightBacktest 回测结果的 HTML 报告生成
"""
import os
from datetime import datetime
from pathlib import Path
from typing import Optional

import pandas as pd
from rs_czsc import WeightBacktest

from .plot_backtest import (
    plot_cumulative_returns,
    plot_drawdown_analysis,
    plot_daily_return_distribution,
    plot_monthly_heatmap,
    get_performance_metrics_cards,
    plot_backtest_stats,
    plot_long_short_comparison
)


def generate_backtest_report(
    df: pd.DataFrame,
    output_path: Optional[str] = None,
    title: str = "权重回测报告",
    **kwargs
) -> str:
    """生成权重回测的 HTML 报告
    
    :param df: pd.DataFrame, 包含 dt, symbol, weight, price 列的权重数据
    :param output_path: HTML 文件输出路径，默认为当前目录下的 backtest_report.html
    :param title: 报告标题
    :param kwargs: 回测参数和显示控制
        - fee_rate: float, 单边交易成本，默认 0.0002
        - digits: int, 权重小数位数，默认 2
        - weight_type: str, 权重类型，默认 'ts'
        - yearly_days: int, 年交易日天数，默认 252
        - n_jobs: int, 并行进程数，默认 1
    :return: HTML 文件路径
    """
    if output_path is None:
        output_path = os.path.join(os.getcwd(), "backtest_report.html")
    
    # 提取回测参数
    fee_rate = kwargs.get("fee_rate", 0.0002)
    digits = kwargs.get("digits", 2)
    weight_type = kwargs.get("weight_type", "ts")
    yearly_days = kwargs.get("yearly_days", 252)
    n_jobs = kwargs.get("n_jobs", 1)
    
    # 创建 WeightBacktest 对象
    wb = WeightBacktest(
        dfw=df,
        fee_rate=fee_rate,
        digits=digits,
        weight_type=weight_type,
        yearly_days=yearly_days,
        n_jobs=n_jobs
    )
    
    # 提取数据
    metrics = _extract_metrics(wb)
    charts = _generate_charts(wb, df, fee_rate, digits, weight_type, yearly_days)
    
    # 生成 HTML
    html_content = _build_html_template(
        report_title=title,
        start_date=df["dt"].min().strftime("%Y-%m-%d"),
        end_date=df["dt"].max().strftime("%Y-%m-%d"),
        fee=fee_rate * 10000,
        digits=digits,
        yearly_days=yearly_days,
        num_symbols=df["symbol"].nunique(),
        metrics=metrics,
        charts=charts
    )
    
    with open(output_path, "w", encoding="utf-8") as f:
        f.write(html_content)
    
    return output_path


def _build_html_template(
    report_title: str,
    start_date: str,
    end_date: str,
    fee: float,
    digits: int,
    yearly_days: int,
    num_symbols: int,
    metrics: list,
    charts: dict
) -> str:
    """构建完整的 HTML 报告
    
    :param report_title: 报告标题
    :param start_date: 开始日期
    :param end_date: 结束日期
    :param fee: 手续费
    :param digits: 权重小数位数
    :param yearly_days: 年交易日天数
    :param num_symbols: 标的数
    :param metrics: 绩效指标列表
    :param charts: 图表字典
    :return: HTML 字符串
    """
    # 生成指标卡片 HTML
    metrics_html = "\n".join([
        f'''                <div class="col-6 col-md-4 col-lg-3 col-xl-2">
                    <div class="metric-card">
                        <div class="metric-value {"metric-positive" if m["is_positive"] else "metric-negative"}">
                            {m["value"]}
                        </div>
                        <div class="metric-label">{m["label"]}</div>
                    </div>
                </div>'''
        for m in metrics
    ])
    
    # 获取当前时间
    current_time = datetime.now().strftime("%Y-%m-%d %H:%M:%S")
    
    # 构建完整 HTML
    html_content = f"""<!DOCTYPE html>
<html lang="zh-CN">
<head>
    <meta charset="UTF-8">
    <meta name="viewport" content="width=device-width, initial-scale=1.0">
    <title>{report_title}</title>
    
    <link href="https://cdn.jsdelivr.net/npm/bootstrap@5.3.0/dist/css/bootstrap.min.css" rel="stylesheet">
    <link rel="stylesheet" href="https://cdn.jsdelivr.net/npm/bootstrap-icons@1.11.0/font/bootstrap-icons.css">
    
    <style>
        :root {{
            --bg-primary: #ffffff;
            --bg-secondary: #f8f9fa;
            --bg-tertiary: #e9ecef;
            --text-primary: #212529;
            --text-secondary: #6c757d;
            --accent-green: #28a745;
            --accent-red: #dc3545;
            --accent-blue: #007bff;
            --accent-yellow: #ffc107;
            --border-color: #dee2e6;
            --shadow: 0 2px 4px rgba(0, 0, 0, 0.1);
        }}
        
        * {{
            margin: 0;
            padding: 0;
            box-sizing: border-box;
        }}
        
        html, body {{
            width: 100%;
            height: 100%;
            overflow-x: hidden;
        }}
        
        body {{
            background-color: var(--bg-primary);
            color: var(--text-primary);
            font-family: 'Segoe UI', system-ui, -apple-system, sans-serif;
            line-height: 1.5;
            display: flex;
            flex-direction: column;
        }}
        
        .header-section {{
            background: linear-gradient(135deg, var(--bg-secondary) 0%, var(--bg-tertiary) 100%);
            border-bottom: 1px solid var(--border-color);
            padding: 1.5rem 0;
            margin-bottom: 1.5rem;
        }}
        
        .header-title {{
            font-size: 1.8rem;
            font-weight: 700;
            color: var(--text-primary);
            margin-bottom: 0.3rem;
        }}
        
        .header-subtitle {{
            color: var(--text-secondary);
            font-size: 0.9rem;
        }}
        
        .param-badges {{
            display: flex;
            flex-wrap: wrap;
            gap: 0.5rem;
            margin-top: 1rem;
        }}
        
        .param-badge {{
            background-color: var(--bg-primary);
            color: var(--text-secondary);
            padding: 0.35rem 0.7rem;
            border-radius: 6px;
            font-size: 0.8rem;
            border: 1px solid var(--border-color);
            transition: all 0.2s;
        }}
        
        .param-badge:hover {{
            border-color: var(--accent-blue);
            color: var(--text-primary);
        }}
        
        .main-content {{
            flex: 1;
            padding: 0 0.5rem 0.5rem 0.5rem;
        }}
        
        .container {{
            max-width: 100%;
            padding: 0 0.5rem;
        }}
        
        .metric-card {{
            background: var(--bg-primary);
            border: 1px solid var(--border-color);
            border-radius: 10px;
            padding: 1rem;
            text-align: center;
            transition: all 0.3s;
            box-shadow: var(--shadow);
            height: 100%;
        }}
        
        .metric-card:hover {{
            transform: translateY(-3px);
            box-shadow: 0 4px 12px rgba(0, 0, 0, 0.15);
            border-color: var(--accent-blue);
        }}
        
        .metric-value {{
            font-size: 1.6rem;
            font-weight: 700;
            margin-bottom: 0.2rem;
        }}
        
        .metric-label {{
            font-size: 0.75rem;
            color: var(--text-secondary);
            text-transform: uppercase;
            letter-spacing: 0.5px;
        }}
        
        .metric-positive {{
            color: var(--accent-green);
        }}
        
        .metric-negative {{
            color: var(--accent-red);
        }}
        
        .metric-neutral {{
            color: var(--accent-blue);
        }}
        
        .chart-card {{
            background: var(--bg-primary);
            border: 1px solid var(--border-color);
            border-radius: 10px;
            overflow: hidden;
            box-shadow: var(--shadow);
            height: calc(100vh - 200px);
        }}
        
        .chart-header {{
            background: var(--bg-secondary);
            padding: 0.8rem 1.2rem;
            border-bottom: 1px solid var(--border-color);
        }}
        
        .chart-title {{
            font-size: 1.1rem;
            font-weight: 600;
            margin: 0;
            color: var(--text-primary);
        }}
        
        .chart-body {{
            padding: 0;
            height: calc(100% - 56px);
            width: 100%;
        }}
        
        .chart-body .plotly-graph-div {{
            width: 100% !important;
            height: 100% !important;
        }}
        
        /* Fix for Plotly layout issues */
        .js-plotly-plot *, .plot-container * {{
            box-sizing: content-box;
        }}
        
        .nav-tabs {{
            border-bottom: 2px solid var(--border-color);
            background: var(--bg-secondary);
        }}
        
        .nav-tabs .nav-link {{
            color: var(--text-secondary);
            border: none;
            border-bottom: 3px solid transparent;
            padding: 0.8rem 1.2rem;
            transition: all 0.2s;
            font-size: 0.9rem;
        }}
        
        .nav-tabs .nav-link:hover {{
            color: var(--text-primary);
            background: var(--bg-tertiary);
        }}
        
        .nav-tabs .nav-link.active {{
            color: var(--accent-blue);
            background: var(--bg-primary);
            border-bottom-color: var(--accent-blue);
        }}
        
        .tab-content {{
            background: var(--bg-primary);
            padding: 0;
            height: 100%;
        }}
        
        .tab-pane {{
            height: 100%;
        }}
        
        .data-table {{
            background: var(--bg-primary);
            border-radius: 10px;
            overflow: hidden;
            box-shadow: var(--shadow);
        }}
        
        .table {{
            color: var(--text-primary);
            margin-bottom: 0;
            font-size: 0.9rem;
        }}
        
        .table thead th {{
            background: var(--bg-secondary);
            border-bottom: 2px solid var(--border-color);
            color: var(--text-primary);
            font-weight: 600;
            padding: 0.8rem;
            text-transform: uppercase;
            font-size: 0.75rem;
        }}
        
        .table tbody tr {{
            border-bottom: 1px solid var(--border-color);
            transition: background 0.2s;
        }}
        
        .table tbody tr:hover {{
            background: var(--bg-secondary);
        }}
        
        .table tbody td {{
            padding: 0.8rem;
            vertical-align: middle;
        }}
        
        .section-header {{
            display: flex;
            align-items: center;
            margin-bottom: 1rem;
            padding-bottom: 0.3rem;
            border-bottom: 2px solid var(--border-color);
        }}
        
        .section-title {{
            font-size: 1.3rem;
            font-weight: 700;
            color: var(--text-primary);
            margin: 0;
        }}
        
        .section-icon {{
            margin-right: 0.5rem;
            color: var(--accent-blue);
        }}
        
        .footer {{
            background: var(--bg-secondary);
            border-top: 1px solid var(--border-color);
            padding: 1rem 0;
            margin-top: auto;
            text-align: center;
            color: var(--text-secondary);
            font-size: 0.85rem;
        }}
        
        @media (max-width: 768px) {{
            .header-title {{
                font-size: 1.4rem;
            }}
            
            .metric-value {{
                font-size: 1.3rem;
            }}
            
            .nav-tabs .nav-link {{
                padding: 0.6rem 0.8rem;
                font-size: 0.85rem;
            }}
            
            .chart-card {{
                height: 70vh;
            }}
            
            .main-content {{
                padding: 0 0.5rem 0.5rem 0.5rem;
            }}
        }}
        
        @media (min-width: 1400px) {{
            .container {{
                max-width: 100%;
            }}
        }}
    </style>
</head>
<body>
    <!-- 头部区域 -->
    <div class="header-section">
        <div class="container">
            <div class="row">
                <div class="col-12">
                    <h1 class="header-title">
                        <i class="bi bi-graph-up-arrow section-icon"></i>
                        {report_title}
                    </h1>
                    <p class="header-subtitle">基于权重策略的回测分析与绩效评估</p>
                    
                    <div class="param-badges">
                        <span class="param-badge">
                            <i class="bi bi-calendar3"></i> {start_date} ~ {end_date}
                        </span>
                        <span class="param-badge">
                            <i class="bi bi-cash-coin"></i> 手续费: {fee:.2f} BP
                        </span>
                        <span class="param-badge">
                            <i class="bi bi-percent"></i> 小数位: {digits}
                        </span>
                        <span class="param-badge">
                            <i class="bi bi-calendar-week"></i> 年交易日: {yearly_days}
                        </span>
                        <span class="param-badge">
                            <i class="bi bi-collection"></i> 标的数: {num_symbols}
                        </span>
                    </div>
                </div>
            </div>
        </div>
    </div>
    
    <!-- 主要内容区域 -->
    <div class="main-content">
        <div class="container">
            <!-- 核心绩效指标 -->
            <section class="mb-4">
                <div class="section-header">
                    <i class="bi bi-speedometer2 section-icon"></i>
                    <h2 class="section-title">核心绩效指标</h2>
                </div>
                
                <div class="row g-2">
{metrics_html}
                </div>
            </section>
            
            <!-- 图表展示区域 -->
            <section class="mb-4">
                <div class="section-header">
                    <i class="bi bi-bar-chart-line section-icon"></i>
                    <h2 class="section-title">可视化分析</h2>
                </div>
                
                <div class="chart-card">
                    <ul class="nav nav-tabs" id="chartTabs" role="tablist">
                        <li class="nav-item">
                            <button class="nav-link active" data-bs-toggle="tab" data-bs-target="#stats" type="button" role="tab">
                                <i class="bi bi-grid-1x2"></i> 回测统计
                            </button>
                        </li>
                        <li class="nav-item">
                            <button class="nav-link" data-bs-toggle="tab" data-bs-target="#cumulative" type="button" role="tab">
                                <i class="bi bi-graph-up"></i> 累计收益
                            </button>
                        </li>
                        <li class="nav-item">
                            <button class="nav-link" data-bs-toggle="tab" data-bs-target="#drawdown" type="button" role="tab">
                                <i class="bi bi-graph-down-arrow"></i> 回撤分析
                            </button>
                        </li>
                        <li class="nav-item">
                            <button class="nav-link" data-bs-toggle="tab" data-bs-target="#daily-return" type="button" role="tab">
                                <i class="bi bi-bar-chart"></i> 日收益分布
                            </button>
                        </li>
                        <li class="nav-item">
                            <button class="nav-link" data-bs-toggle="tab" data-bs-target="#monthly-heatmap" type="button" role="tab">
                                <i class="bi bi-grid-3x3-gap"></i> 月度热力图
                            </button>
                        </li>
                        <li class="nav-item">
                            <button class="nav-link" data-bs-toggle="tab" data-bs-target="#long-short" type="button" role="tab">
                                <i class="bi bi-arrows-collapse"></i> 多空对比
                            </button>
                        </li>
                    </ul>
                    
                    <div class="tab-content">
                        <div class="tab-pane fade show active" id="stats" role="tabpanel">
                            <div class="chart-body">
                                {charts["backtest_stats"]}
                            </div>
                        </div>
                        <div class="tab-pane fade" id="cumulative" role="tabpanel">
                            <div class="chart-body">
                                {charts["cumulative_returns"]}
                            </div>
                        </div>
                        <div class="tab-pane fade" id="drawdown" role="tabpanel">
                            <div class="chart-body">
                                {charts["drawdown"]}
                            </div>
                        </div>
                        <div class="tab-pane fade" id="daily-return" role="tabpanel">
                            <div class="chart-body">
                                {charts["daily_return"]}
                            </div>
                        </div>
                        <div class="tab-pane fade" id="monthly-heatmap" role="tabpanel">
                            <div class="chart-body">
                                {charts["monthly_heatmap"]}
                            </div>
                        </div>
                        <div class="tab-pane fade" id="long-short" role="tabpanel">
                            <div class="chart-body">
                                {charts["long_short_comparison"]}
                            </div>
                        </div>
                    </div>
                </div>
            </section>
        </div>
    </div>
    
    <!-- 页脚 -->
    <footer class="footer">
        <div class="container">
            <p class="mb-0">
                <i class="bi bi-code-square"></i>
                由 CZSC 缠中说禅技术分析工具生成 | 
                <i class="bi bi-clock-history"></i>
                生成时间: {current_time}
            </p>
        </div>
    </footer>
    
    <script src="https://cdn.jsdelivr.net/npm/bootstrap@5.3.0/dist/js/bootstrap.bundle.min.js"></script>
    <script>
        // 监听 Tab 切换事件，重新调整图表大小
        document.addEventListener('DOMContentLoaded', function() {{
            var triggerTabList = [].slice.call(document.querySelectorAll('button[data-bs-toggle="tab"]'))
            triggerTabList.forEach(function(triggerEl) {{
                triggerEl.addEventListener('shown.bs.tab', function(event) {{
                    // 获取目标 Tab 面板 ID
                    var targetId = event.target.getAttribute('data-bs-target');
                    var targetPane = document.querySelector(targetId);
                    
                    // 查找该面板内的 Plotly 图表
                    var plotlyDiv = targetPane.querySelector('.plotly-graph-div');
                    if (plotlyDiv) {{
                        Plotly.Plots.resize(plotlyDiv);
                    }}
                }})
            }})
            
            // 窗口大小改变时也触发 resize
            window.addEventListener('resize', function() {{
                var activePane = document.querySelector('.tab-pane.active');
                if (activePane) {{
                    var plotlyDiv = activePane.querySelector('.plotly-graph-div');
                    if (plotlyDiv) {{
                        Plotly.Plots.resize(plotlyDiv);
                    }}
                }}
            }});
        }});
    </script>
</body>
</html>
    """
    
    return html_content


def _extract_metrics(wb: WeightBacktest) -> list:
    """提取核心绩效指标
    
    :param wb: WeightBacktest 对象
    :return: 指标列表，每个元素为 {"label": str, "value": str, "is_positive": bool}
    """
    return get_performance_metrics_cards(wb.stats)


def _generate_charts(wb: WeightBacktest, df: pd.DataFrame, fee_rate: float, 
                     digits: int, weight_type: str, yearly_days: int) -> dict:
    """生成所有图表
    
    :param wb: WeightBacktest 对象
    :param df: 原始权重数据
    :param fee_rate: 手续费率
    :param digits: 权重小数位数
    :param weight_type: 权重类型
    :param yearly_days: 年交易日天数
    :return: 字典，包含所有图表的 HTML 片段
    """
    dret = wb.daily_return.copy()
    dret["dt"] = pd.to_datetime(dret["date"])
    dret = dret.set_index("dt").drop(columns=["date"])
    
    # 生成图表，第一个图表包含 Plotly.js，后续图表不包含
    config = {'responsive': True, 'displayModeBar': True}
    fig_stats = plot_backtest_stats(dret, ret_col="total", title="回测统计概览", template="plotly")
    fig1 = plot_cumulative_returns(dret, title="累计收益曲线", template="plotly")
    fig2 = plot_drawdown_analysis(dret, ret_col="total", title="回撤分析", template="plotly")
    fig3 = plot_daily_return_distribution(dret, ret_col="total", title="日收益分布", template="plotly")
    fig4 = plot_monthly_heatmap(dret, ret_col="total", title="月度收益热力图", template="plotly")
    
    charts = {
        "backtest_stats": fig_stats.to_html(include_plotlyjs=True, full_html=False, config=config),
        "cumulative_returns": fig1.to_html(include_plotlyjs=False, full_html=False, config=config),
        "drawdown": fig2.to_html(include_plotlyjs=False, full_html=False, config=config),
        "daily_return": fig3.to_html(include_plotlyjs=False, full_html=False, config=config),
        "monthly_heatmap": fig4.to_html(include_plotlyjs=False, full_html=False, config=config),
    }
    
    # 生成多空对比图表
    try:
        # 拆分数据创建多个回测（参考 show_long_short_backtest）
        df_base = df[["dt", "symbol", "weight", "price"]].copy()
        
        dfl = df_base.copy()
        dfl["weight"] = dfl["weight"].clip(lower=0)  # 只保留多头
        
        dfs = df_base.copy()
        dfs["weight"] = dfs["weight"].clip(upper=0)  # 只保留空头
        
        dfb = df_base.copy()
        dfb['weight'] = 1  # 基准等权
        
        # 创建多个回测实例
        wbs = {
            "原始策略": WeightBacktest(df_base, fee_rate=fee_rate, digits=digits, 
                                       weight_type=weight_type, yearly_days=yearly_days),
            "策略多头": WeightBacktest(dfl, fee_rate=fee_rate, digits=digits, 
                                      weight_type=weight_type, yearly_days=yearly_days),
            "策略空头": WeightBacktest(dfs, fee_rate=fee_rate, digits=digits, 
                                      weight_type=weight_type, yearly_days=yearly_days),
            "基准等权": WeightBacktest(dfb, fee_rate=fee_rate, digits=digits, 
                                      weight_type="ts", yearly_days=yearly_days),
        }
        
        # 汇总日收益数据（参考 show_multi_backtest）
        dailys = []
        for strategy, wb_obj in wbs.items():
            daily = wb_obj.daily_return.copy()[["date", "total"]]
            daily["strategy"] = strategy
            daily["return"] = daily["total"]
            daily["dt"] = daily["date"]
            dailys.append(daily[["dt", "strategy", "return"]])
        
        dailys = pd.concat(dailys, axis=0)
        dailys["dt"] = pd.to_datetime(dailys["dt"])
        df_dailys = pd.pivot_table(dailys, index="dt", columns="strategy", values="return")
        
        # 汇总绩效指标
        stats_rows = []
        for strategy, wb_obj in wbs.items():
            stats = {"策略名称": strategy}
            stats.update(wb_obj.stats)
            stats_rows.append(stats)
        df_stats = pd.DataFrame(stats_rows)
        
        # 生成多空对比图表
        fig_ls = plot_long_short_comparison(df_dailys, df_stats, title="多空收益对比", template="plotly")
        charts["long_short_comparison"] = fig_ls.to_html(include_plotlyjs=False, full_html=False, config=config)
    except Exception as e:
        # 如果多空对比生成失败，不影响其他图表
        charts["long_short_comparison"] = f"<div style='padding: 20px; text-align: center; color: red;'>多空对比图生成失败: {str(e)}</div>"
    
    return charts






import os
import time
import shutil
import loguru
import hashlib
import requests
import threading
import pandas as pd
from pathlib import Path
from functools import partial, lru_cache
from typing import Optional, Dict, Any


def set_url_token(token, url, **kwargs):
    """设置指定 URL 数据接口的凭证码，通常一台机器只需要设置一次即可

    :param token: 凭证码
    :param url: 数据接口地址
    """
    logger = kwargs.get("logger", loguru.logger)
    hash_key = hashlib.md5(str(url).encode("utf-8")).hexdigest()
    file_token = Path("~").expanduser() / f"{hash_key}.txt"
    with open(file_token, "w", encoding="utf-8") as f:
        f.write(token)
    logger.info(f"{url} 数据访问凭证码已保存到 {file_token}")


def get_url_token(url, **kwargs):
    """获取指定 URL 数据接口的凭证码"""
    logger = kwargs.get("logger", loguru.logger)
    hash_key = hashlib.md5(str(url).encode("utf-8")).hexdigest()
    file_token = Path("~").expanduser() / f"{hash_key}.txt"
    if file_token.exists():
        logger.info(f"从 {file_token} 读取 {url} 的访问凭证码")
        return open(file_token, "r", encoding="utf-8").read()

    logger.warning(f"请设置 {url} 的访问凭证码，如果没有请联系管理员申请")
    token = input(f"请输入 {url} 的访问凭证码（token）：")
    if token:
        set_url_token(token, url)
        return token
    return None


class DataClient:
    """
    数据接口客户端，支持本地缓存，兼容Tushare数据接口。
    支持多线程/多进程安全，详细日志，异常处理。
    """
    __version__ = "V250719"
    _cache_lock = threading.Lock()  # 进程内线程锁，防止并发冲突
    
    @lru_cache(maxsize=128)
    def _get_cache_key(self, req_params_str: str) -> str:
        """缓存哈希计算，避免重复计算"""
        return hashlib.md5(req_params_str.encode('utf-8')).hexdigest().upper()[:8]

    def __init__(self, token: Optional[str] = None, url: str = "http://api.tushare.pro", timeout: int = 300, verbose: bool = False, **kwargs):
        """
        初始化数据客户端。
        :param token: str, API接口TOKEN
        :param url: str, API接口地址
        :param timeout: int, 请求超时时间
        :param verbose: bool, 是否开启详细日志模式，显示请求细节
        :param kwargs: 其他参数（clear_cache, cache_path, logger）
        """
        from czsc.utils.cache import get_dir_size
        self.logger = kwargs.pop("logger", loguru.logger)

        self.__token = token or get_url_token(url, logger=self.logger)
        self.__http_url = url
        self.__timeout = timeout
        self.__url_hash = hashlib.md5(str(url).encode("utf-8")).hexdigest()[:8]
        assert self.__token, "请设置czsc_token凭证码，如果没有请联系管理员申请"
        self.cache_path = Path(kwargs.get("cache_path", os.path.expanduser("~/.quant_data_cache")))
        self.cache_path.mkdir(exist_ok=True, parents=True)
        self.verbose = verbose
        if self.verbose:
            self.logger.info(
                f"数据URL: {url} 数据缓存路径：{self.cache_path} 占用磁盘空间：{get_dir_size(self.cache_path) / 1024 / 1024:.2f} MB"
            )
        if kwargs.get("clear_cache", False):
            self.clear_cache()

    def _get_cache(self, file_cache: Path, api_name: str, kwargs: Dict[str, Any], logger) -> Optional[pd.DataFrame]:
        """读取缓存"""
        if file_cache.exists():
            with self._cache_lock:
                try:
                    df = pd.read_pickle(file_cache)
                    if self.verbose:
                        logger.info(f"缓存命中 | API：{api_name}；参数：{kwargs}；数据量：{df.shape}")
                    return df
                except Exception as e:
                    logger.warning(f"读取缓存文件失败: {file_cache}, 错误: {e}")
        return None

    def _set_cache(self, file_cache: Path, df: pd.DataFrame, logger) -> None:
        """写入缓存"""
        with self._cache_lock:
            try:
                df.to_pickle(file_cache)
            except Exception as e:
                logger.warning(f"写入缓存文件失败: {file_cache}, 错误: {e}")

    def _request_api(self, req_params: Dict[str, Any], api_name: str, kwargs: Dict[str, Any], logger, retries: int = 3) -> Optional[requests.Response]:
        """发起API请求，包含重试机制"""
        for attempt in range(retries):
            try:
                res = requests.post(self.__http_url, json=req_params, timeout=self.__timeout)
                if res.status_code == 200:
                    return res
                else:
                    # 处理非200状态码
                    if attempt == retries - 1:
                        logger.error(f"API请求失败(最后一次重试): {api_name}；参数：{kwargs}；状态码：{res.status_code}；响应：{res.text}")
                        return None
                    else:
                        logger.warning(f"API请求失败(第{attempt+1}次重试): {api_name}；状态码：{res.status_code}；响应：{res.text}")
                        time.sleep(0.5 * (attempt + 1))  # 递增延迟
                        continue
                
            except requests.RequestException as e:
                if attempt == retries - 1:
                    logger.error(f"请求API失败(最后一次重试): {api_name}；参数：{kwargs}；错误: {e}")
                    return None
                else:
                    logger.warning(f"请求API失败(第{attempt+1}次重试): {api_name}；错误: {e}")
                    time.sleep(0.5 * (attempt + 1))  # 递增延迟
                    
            except Exception as e:
                logger.error(f"请求API失败: {api_name}；参数：{kwargs}；错误: {e}")
                return None
        
        logger.error(f"API请求失败，所有重试都失败")
        return None

    def _validate_response(self, res: requests.Response, api_name: str, kwargs: Dict[str, Any], logger) -> Optional[Dict[str, Any]]:
        """校验API返回结构
        
        结构说明：
        {
            "code": 0,
            "data": {
                "items": [
                    {
                        "field1": "value1",
                        "field2": "value2"
                    }
                ],
                "fields": ["field1", "field2"]
            },
            "message": "success"
        }
        """
        try:
            result = res.json()
        except Exception as e:
            logger.error(f"API返回非JSON格式: {e}")
            return None
        if not isinstance(result, dict) or "code" not in result:
            logger.error(f"API返回结构异常: {result}")
            return None
        if result["code"] != 0:
            logger.error(f"API: {api_name} - {kwargs} 数据获取失败: {result}")
            return None
        data = result.get("data", {})
        if not isinstance(data, dict) or "items" not in data or "fields" not in data:
            logger.error(f"API返回data结构异常: {data}")
            return None
        return data

    def _build_dataframe(self, data: Dict[str, Any], logger) -> pd.DataFrame:
        """构建DataFrame"""
        try:
            df = pd.DataFrame(data["items"], columns=data["fields"])
            return df
        except Exception as e:
            self.logger.error(f"DataFrame构建失败: {e}")
            return pd.DataFrame()

    def clear_cache(self, **kwargs) -> None:
        """
        清空缓存目录。加锁防止并发冲突。
        """
        logger = kwargs.pop("logger", self.logger)
        with self._cache_lock:
            if self.cache_path.exists():
                try:
                    shutil.rmtree(self.cache_path)
                except Exception as e:
                    logger.warning(f"清空缓存时出错: {e}")
            if self.verbose:
                logger.info(f"{self.cache_path} 路径下的数据缓存已清空")
            self.cache_path.mkdir(exist_ok=True, parents=True)

    def clear_api_cache(self, api_name: str, expiration: int = -1) -> None:
        """
        清除指定API的缓存文件，过期时间单位秒。
        """
        path = self.cache_path / f"{self.__url_hash}_{api_name}"
        if path.exists() and expiration > 0:
            with self._cache_lock:
                for file in path.glob("*"):
                    if time.time() - file.stat().st_mtime > expiration:
                        try:
                            file.unlink()
                            if self.verbose:
                                self.logger.info(f"清除缓存文件: {file}")
                        except Exception as e:
                            self.logger.error(f"删除缓存文件失败: {file}, 错误: {e}")
        else:
            if self.verbose:
                self.logger.warning(f"未找到缓存目录: {path}")

    def post_request(self, api_name: str, fields: str = "", **kwargs) -> pd.DataFrame:
        """
        执行API数据查询，主流程调度。
        """
        if not api_name or not isinstance(api_name, str):
            raise ValueError(f"无效的api_name: {api_name}")

        logger = self.logger
        v = kwargs.get("v", 2)
        stime = time.time()
        if api_name in ["__getstate__", "__setstate__"]:
            return pd.DataFrame()

        ttl = int(kwargs.pop("ttl", -1))
        req_params = {
            "api_name": api_name,
            "token": self.__token,
            "params": kwargs,
            "fields": fields,
            "v": v
        }
        path = self.cache_path / f"{self.__url_hash}_{api_name}"
        path.mkdir(exist_ok=True, parents=True)
        cache_key = self._get_cache_key(str(req_params))
        file_cache = path / f"{cache_key}.pkl"

        # 读取缓存
        if file_cache.exists() and (ttl == -1 or time.time() - file_cache.stat().st_mtime < ttl):
            df = self._get_cache(file_cache, api_name, kwargs, logger)
            if df is not None:
                return df

        # 网络请求
        res = self._request_api(req_params, api_name, kwargs, logger)
        if res is None:
            return pd.DataFrame()

        # 校验返回结构
        data = self._validate_response(res, api_name, kwargs, logger)
        if data is None:
            return pd.DataFrame()

        # 构建DataFrame
        df = self._build_dataframe(data, logger)
        if df.empty:
            return df

        # 写缓存
        self._set_cache(file_cache, df, logger)

        if self.verbose:
            logger.info(f"本次获取数据总耗时：{time.time() - stime:.2f}秒；API：{api_name}；参数：{kwargs}；数据量：{df.shape}")
        return df

    def __getattr__(self, name):
        """
        动态API方法调用，等价于 self.post_request(name, ...)
        """
        return partial(self.post_request, name)




"""
持仓权重分析相关的可视化组件

包含权重时序分析、分布分析、累积分布函数、绝对仓位分析等功能
"""
import streamlit as st
import plotly.graph_objects as go
from .base import generate_component_key


def show_weight_ts(df, key=None, **kwargs):
    """展示权重时序分析
    
    :param df: 包含dt、symbol、weight列的DataFrame
    :param key: str, 可选，组件的唯一标识符，默认自动生成
    :param kwargs:
        - title: str, 图表标题，默认为 "策略持仓权重分布分析"
        - height: int, 图表高度，默认为 800
        - show_position_count: bool, 是否显示持仓数量，默认为 True
    """
    from czsc.utils.plot_weight import plot_weight_time_series
    
    title = kwargs.get("title", "策略持仓权重分布分析")
    height = kwargs.get("height", 800)
    show_position_count = kwargs.get("show_position_count", True)
    
    fig = plot_weight_time_series(df, title=title, height=height, show_position_count=show_position_count)
    
    # 修复暗色主题支持
    fig.update_layout(
        plot_bgcolor=None,  # 使用 Streamlit 主题
        paper_bgcolor=None,  # 使用 Streamlit 主题
        font=dict(color=None)  # 使用 Streamlit 主题
    )
    
    if key is None:
        key = generate_component_key(df, prefix="w_ts", title=title, show_position_count=show_position_count, height=height)
    
    st.plotly_chart(fig, key=key, width='stretch')


def show_weight_dist(df, key=None, **kwargs):
    """展示权重分布分析（直方图与核密度估计）
    
    :param df: 包含dt、symbol、weight列的DataFrame
    :param key: str, 可选，组件的唯一标识符，默认自动生成
    :param kwargs:
        - title: str, 图表标题，默认为 "仓位分布直方图与核密度估计"
        - height: int, 图表高度，默认为 800
        - width: int, 图表宽度，默认为 900
    """
    from czsc.utils.plot_weight import plot_weight_histogram_kde
    
    title = kwargs.get("title", "仓位分布直方图与核密度估计")
    height = kwargs.get("height", 800)
    width = kwargs.get("width", 900)
    
    fig = plot_weight_histogram_kde(df, title=title, height=height, width=width)
    
    # 修复暗色主题支持
    fig.update_layout(
        plot_bgcolor=None,  # 使用 Streamlit 主题
        paper_bgcolor=None,  # 使用 Streamlit 主题
        font=dict(color=None)  # 使用 Streamlit 主题
    )
    
    if key is None:
        key = generate_component_key(df, prefix="w_dist", title=title, height=height, width=width)
    
    st.plotly_chart(fig, key=key, width='stretch')


def show_weight_cdf(df, key=None, **kwargs):
    """展示权重累积分布函数(CDF)
    
    :param df: 包含dt、symbol、weight列的DataFrame
    :param key: str, 可选，组件的唯一标识符，默认自动生成
    :param kwargs:
        - title: str, 图表标题，默认为 "仓位分布累积分布函数(CDF)对比"
        - height: int, 图表高度，默认为 600
        - width: int, 图表宽度，默认为 800
        - show_percentiles: bool, 是否显示分位数参考线，默认为 True
    """
    from czsc.utils.plot_weight import plot_weight_cdf
    
    title = kwargs.get("title", "仓位分布累积分布函数(CDF)对比")
    height = kwargs.get("height", 600)
    width = kwargs.get("width", 800)
    show_percentiles = kwargs.get("show_percentiles", True)
    
    fig = plot_weight_cdf(df, title=title, height=height, width=width, show_percentiles=show_percentiles)
    
    # 修复暗色主题支持
    fig.update_layout(
        plot_bgcolor=None,  # 使用 Streamlit 主题
        paper_bgcolor=None,  # 使用 Streamlit 主题
        font=dict(color=None)  # 使用 Streamlit 主题
    )
    
    if key is None:
        key = generate_component_key(df, prefix="w_cdf", title=title, show_percentiles=show_percentiles, height=height, width=width)
    
    st.plotly_chart(fig, key=key, width='stretch')


def show_weight_abs(df, key=None, **kwargs):
    """展示绝对仓位详细分析
    
    :param df: 包含dt、symbol、weight列的DataFrame
    :param key: str, 可选，组件的唯一标识符，默认自动生成
    :param kwargs:
        - title: str, 图表标题，默认为 "绝对仓位详细分析图表"
        - height: int, 图表高度，默认为 900
        - width: int, 图表宽度，默认为 1000
        - ma_windows: list, 移动平均线窗口列表，默认为 [5, 20, 60]
        - volatility_window: int, 波动率计算窗口，默认为 20
    """
    from czsc.utils.plot_weight import plot_absolute_position_analysis
    
    title = kwargs.get("title", "绝对仓位详细分析图表")
    height = kwargs.get("height", 900)
    width = kwargs.get("width", 1000)
    ma_windows = kwargs.get("ma_windows", [5, 20, 60])
    volatility_window = kwargs.get("volatility_window", 20)
    
    fig = plot_absolute_position_analysis(
        df, title=title, height=height, width=width, 
        ma_windows=ma_windows, volatility_window=volatility_window
    )
    
    # 修复暗色主题支持
    fig.update_layout(
        plot_bgcolor=None,  # 使用 Streamlit 主题
        paper_bgcolor=None,  # 使用 Streamlit 主题
        font=dict(color=None)  # 使用 Streamlit 主题
    )
    
    if key is None:
        key = generate_component_key(df, prefix="w_abs", title=title, ma_windows=ma_windows, volatility_window=volatility_window, height=height, width=width)
    
    st.plotly_chart(fig, key=key, width='stretch')


__all__ = [
    'show_weight_ts',
    'show_weight_dist',
    'show_weight_cdf',
    'show_weight_abs',
]



"""
历史功能代码存档

主要是一些已经用 rust 复现的功能需要在这里做个存档
"""

from .enum import Operate, Freq, Mark, Direction
from .analyze import CZSC, remove_include, check_bi, check_fx, check_fxs
from .bar_generator import BarGenerator, freq_end_time, is_trading_time, format_standard_kline
from .objects import RawBar, NewBar, FX, BI, FakeBI, ZS, Signal, Event, Position
from .weight_backtest import WeightBacktest


__all__ = [
    "Operate", "Freq", "Mark", "Direction",
    "CZSC", "remove_include", "check_bi", "check_fx", "check_fxs",
    "BarGenerator", "freq_end_time", "is_trading_time",
    "RawBar", "NewBar", "FX", "BI", "FakeBI", "ZS", "Signal", "Event", "Position",
    "WeightBacktest", "format_standard_kline"
]


"""
权重回测可视化 Plotly 绘图函数

从 czsc.svc 模块中提取的 WeightBacktest 相关的绘图代码，按功能整理
"""

import pandas as pd
import plotly.graph_objects as go
import plotly.express as px
from plotly.subplots import make_subplots
from typing import Union


def plot_cumulative_returns(
    dret: pd.DataFrame, 
    title: str = "累计收益",
    template: str = "plotly",
    to_html: bool = False,
    include_plotlyjs: bool = True
) -> Union[go.Figure, str]:
    """绘制累计收益曲线
    
    参考 czsc.svc.returns.show_cumulative_returns
    
    :param dret: 日收益数据，index为日期，columns为策略收益
    :param title: 图表标题
    :param template: plotly 模板名称，默认 'plotly'（白色主题），可选 'plotly_dark'（深色主题）
    :param to_html: 是否转换为 HTML，默认 False（返回 Figure 对象）
    :param include_plotlyjs: 转换 HTML 时是否包含 plotly.js 库，默认 True
    :return: Figure 对象或 HTML 字符串
    """
    assert dret.index.dtype == "datetime64[ns]", "index必须是datetime64[ns]类型"
    assert dret.index.is_unique, "df 的索引必须唯一"
    assert dret.index.is_monotonic_increasing, "df 的索引必须单调递增"
    
    df_cumsum = dret.cumsum()
    fig = px.line(
        df_cumsum, 
        y=df_cumsum.columns.to_list(), 
        title=title,
        color_discrete_sequence=px.colors.qualitative.Plotly
    )
    fig.update_xaxes(title="")
    
    # 添加年度分隔线
    years = df_cumsum.index.year.unique()
    for year in years:
        first_date = df_cumsum[df_cumsum.index.year == year].index.min()
        fig.add_vline(x=first_date, line_dash="dash", line_color="red")
    
    fig.update_layout(
        hovermode="x unified",
        template=template,
        margin=dict(l=0, r=0, b=0, t=40),
        legend=dict(orientation="h", y=-0.1, xanchor="center", x=0.5)
    )
    
    # 仅显示 total 的曲线，其他的曲线隐藏
    for trace in fig.data:
        if trace.name != 'total':
            trace.visible = 'legendonly'
    
    if to_html:
        return fig.to_html(include_plotlyjs=include_plotlyjs, full_html=False)
    return fig


def plot_drawdown_analysis(
    dret: pd.DataFrame, 
    ret_col: str = "total", 
    title: str = "回撤分析",
    template: str = "plotly",
    to_html: bool = False,
    include_plotlyjs: bool = True
) -> Union[go.Figure, str]:
    """绘制回撤分析图
    
    参考 czsc.svc.returns.show_drawdowns
    
    :param dret: 日收益数据，包含 ret_col 列
    :param ret_col: 收益列名
    :param title: 图表标题
    :param template: plotly 模板名称，默认 'plotly'（白色主题），可选 'plotly_dark'（深色主题）
    :param to_html: 是否转换为 HTML，默认 False（返回 Figure 对象）
    :param include_plotlyjs: 转换 HTML 时是否包含 plotly.js 库，默认 True
    :return: Figure 对象或 HTML 字符串
    """
    df = dret[[ret_col]].copy().fillna(0).sort_index(ascending=True)
    
    # 计算回撤数据
    df["cum_ret"] = df[ret_col].cumsum()
    df["cum_max"] = df["cum_ret"].cummax()
    df["drawdown"] = df["cum_ret"] - df["cum_max"]
    
    fig = go.Figure()
    
    # 回撤曲线
    fig.add_trace(go.Scatter(
        x=df.index, 
        y=df["drawdown"] * 100,  # 转为百分比
        fillcolor="salmon", 
        line=dict(color="salmon"),
        fill="tozeroy",
        mode="lines", 
        name="回撤", 
        opacity=0.5
    ))
    
    # 累计收益曲线
    fig.add_trace(go.Scatter(
        x=df.index, 
        y=df["cum_ret"], 
        mode="lines", 
        name="累计收益", 
        yaxis="y2", 
        opacity=0.8, 
        line=dict(color="#34a853")
    ))
    
    fig.update_layout(
        yaxis2=dict(title="累计收益", overlaying="y", side="right"),
        margin=dict(l=0, r=0, t=0, b=0), 
        title=title, 
        xaxis_title="", 
        yaxis_title="净值回撤 (%)", 
        legend_title="回撤分析",
        hovermode="x unified",
        template=template
    )
    
    # 添加分位数线
    for q in [0.1, 0.3, 0.5]:
        y1 = df["drawdown"].quantile(q)
        fig.add_hline(y=y1 * 100, line_dash="dot", line_color="rgba(52,168,83,0.5)", line_width=1)
    
    if to_html:
        return fig.to_html(include_plotlyjs=include_plotlyjs, full_html=False)
    return fig


def plot_daily_return_distribution(
    dret: pd.DataFrame, 
    ret_col: str = "total", 
    title: str = "日收益分布",
    template: str = "plotly",
    to_html: bool = False,
    include_plotlyjs: bool = True
) -> Union[go.Figure, str]:
    """绘制日收益分布直方图
    
    :param dret: 日收益数据，包含 ret_col 列
    :param ret_col: 收益列名
    :param title: 图表标题
    :param template: plotly 模板名称，默认 'plotly'（白色主题），可选 'plotly_dark'（深色主题）
    :param to_html: 是否转换为 HTML，默认 False（返回 Figure 对象）
    :param include_plotlyjs: 转换 HTML 时是否包含 plotly.js 库，默认 True
    :return: Figure 对象或 HTML 字符串
    """
    daily_returns = (dret[ret_col] * 100).reset_index()
    daily_returns.columns = ["dt", "日收益"]
    
    fig = px.histogram(daily_returns, x="日收益", nbins=50, title=title)
    fig.update_xaxes(title="日收益 (%)")
    fig.update_yaxes(title="频数")
    
    fig.update_layout(
        template=template,
        margin=dict(l=0, r=0, b=0, t=40),
        bargap=0.1,
        hovermode="x unified"
    )
    
    if to_html:
        return fig.to_html(include_plotlyjs=include_plotlyjs, full_html=False)
    return fig


def plot_monthly_heatmap(
    dret: pd.DataFrame, 
    ret_col: str = "total", 
    title: str = "月度收益热力图",
    template: str = "plotly",
    to_html: bool = False,
    include_plotlyjs: bool = True
) -> Union[go.Figure, str]:
    """绘制月度收益热力图
    
    :param dret: 日收益数据，index为日期
    :param ret_col: 收益列名
    :param title: 图表标题
    :param template: plotly 模板名称，默认 'plotly'（白色主题），可选 'plotly_dark'（深色主题）
    :param to_html: 是否转换为 HTML，默认 False（返回 Figure 对象）
    :param include_plotlyjs: 转换 HTML 时是否包含 plotly.js 库，默认 True
    :return: Figure 对象或 HTML 字符串
    """
    df = dret[[ret_col]].copy()
    df['year'] = df.index.year
    df['month'] = df.index.month
    
    # 计算月度收益
    monthly_ret = df.groupby(['year', 'month'])[ret_col].sum() * 100
    monthly_ret = monthly_ret.reset_index()
    monthly_ret.columns = ['年份', '月份', '日收益']
    
    # 创建透视表
    pivot_table = monthly_ret.pivot(index='年份', columns='月份', values='日收益')
    
    fig = go.Figure(data=go.Heatmap(
        z=pivot_table.values,
        x=['1月', '2月', '3月', '4月', '5月', '6月', 
           '7月', '8月', '9月', '10月', '11月', '12月'],
        y=pivot_table.index,
        colorscale='RdYlGn',
        colorbar=dict(title="日收益 (%)")
    ))
    
    fig.update_layout(
        title=title,
        xaxis_title="月份",
        yaxis_title="年份",
        template=template,
        margin=dict(l=0, r=0, b=0, t=40)
    )
    
    if to_html:
        return fig.to_html(include_plotlyjs=include_plotlyjs, full_html=False)
    return fig


def get_performance_metrics_cards(stats: dict) -> list:
    """从WeightBacktest.stats中提取核心绩效指标，用于HTML报告中的指标卡
    
    :param stats: WeightBacktest.stats 字典
    :return: 指标列表
    """
    metrics = [
        {"label": "年化收益率", "value": f"{stats.get('年化', 0):.2%}", "is_positive": stats.get('年化', 0) > 0},
        {"label": "单笔收益(BP)", "value": f"{stats.get('单笔收益', 0):.2f}", "is_positive": stats.get('单笔收益', 0) > 0},
        {"label": "交易胜率", "value": f"{stats.get('交易胜率', 0):.2%}", "is_positive": stats.get('交易胜率', 0) > 0.5},
        {"label": "持仓K线数", "value": f"{stats.get('持仓K线数', 0):.0f}", "is_positive": True},
        {"label": "最大回撤", "value": f"{stats.get('最大回撤', 0):.2%}", "is_positive": stats.get('最大回撤', 0) < 0.1},
        {"label": "年化", "value": f"{stats.get('年化', 0):.2%}", "is_positive": stats.get('年化', 0) > 0},
        {"label": "夏普", "value": f"{stats.get('夏普', 0):.2f}", "is_positive": stats.get('夏普', 0) > 1},
        {"label": "卡玛", "value": f"{stats.get('卡玛', 0):.2f}", "is_positive": stats.get('卡玛', 0) > 1},
        {"label": "年化波动率", "value": f"{stats.get('年化波动率', 0):.2%}", "is_positive": stats.get('年化波动率', 0) < 0.2},
        {"label": "多头占比", "value": f"{stats.get('多头占比', 0):.2%}", "is_positive": True},
        {"label": "空头占比", "value": f"{stats.get('空头占比', 0):.2%}", "is_positive": True},
    ]
    return metrics

def plot_backtest_stats(
    dret: pd.DataFrame, 
    ret_col: str = "total", 
    title: str = "回测统计概览",
    template: str = "plotly",
    to_html: bool = False,
    include_plotlyjs: bool = True
) -> Union[go.Figure, str]:
    """绘制回测统计概览图（4宫格）
    
    :param dret: 日收益数据，index为日期
    :param ret_col: 收益列名
    :param title: 图表标题
    :param template: plotly 模板名称
    :param to_html: 是否转换为 HTML
    :param include_plotlyjs: 转换 HTML 时是否包含 plotly.js 库
    :return: Figure 对象或 HTML 字符串
    """
    # 创建 2x2 子图
    fig = make_subplots(
        rows=2, cols=2,
        subplot_titles=("累计收益", "回撤分析", "日收益分布", "月度收益热力图"),
        vertical_spacing=0.1,
        horizontal_spacing=0.05,
        specs=[[{"secondary_y": False}, {"secondary_y": True}],
               [{"secondary_y": False}, {"secondary_y": False}]]
    )
    
    # 1. 累计收益 (Top Left)
    df_cumsum = dret[[ret_col]].cumsum()
    fig.add_trace(
        go.Scatter(x=df_cumsum.index, y=df_cumsum[ret_col], name="累计收益", 
                   mode='lines', line=dict(color="#34a853"),
                   legendgroup="1"),
        row=1, col=1
    )
    
    # 2. 回撤分析 (Top Right)
    df = dret[[ret_col]].copy().fillna(0).sort_index(ascending=True)
    df["cum_ret"] = df[ret_col].cumsum()
    df["cum_max"] = df["cum_ret"].cummax()
    df["drawdown"] = df["cum_ret"] - df["cum_max"]
    
    fig.add_trace(
        go.Scatter(x=df.index, y=df["drawdown"] * 100, fillcolor="salmon", 
                   line=dict(color="salmon"), fill="tozeroy", mode="lines", 
                   name="回撤", opacity=0.5, legendgroup="2"),
        row=1, col=2
    )
    fig.add_trace(
        go.Scatter(x=df.index, y=df["cum_ret"], mode="lines", name="累计收益", 
                   opacity=0.8, line=dict(color="#34a853"), legendgroup="2"),
        row=1, col=2, secondary_y=True
    )
    
    # 3. 日收益分布 (Bottom Left)
    daily_returns = (dret[ret_col] * 100)
    fig.add_trace(
        go.Histogram(x=daily_returns, nbinsx=50, name="日收益分布", showlegend=False),
        row=2, col=1
    )
    
    # 4. 月度收益热力图 (Bottom Right)
    df['year'] = df.index.year
    df['month'] = df.index.month
    monthly_ret = df.groupby(['year', 'month'])[ret_col].sum() * 100
    monthly_ret = monthly_ret.reset_index()
    monthly_ret.columns = ['年份', '月份', '日收益']
    pivot_table = monthly_ret.pivot(index='年份', columns='月份', values='日收益')
    
    fig.add_trace(
        go.Heatmap(
            z=pivot_table.values,
            x=['1月', '2月', '3月', '4月', '5月', '6月', 
               '7月', '8月', '9月', '10月', '11月', '12月'],
            y=pivot_table.index,
            colorscale='RdYlGn',
            showscale=True,
            colorbar=dict(title="收益(%)", len=0.45, y=0.2),
            name="月度热力图"
        ),
        row=2, col=2
    )
    
    # 更新布局
    fig.update_layout(
        title=title,
        template=template,
        height=800,
        margin=dict(l=20, r=20, b=20, t=60),
        hovermode="x unified",
        showlegend=True
    )
    
    # 更新坐标轴标签
    fig.update_yaxes(title_text="累计收益", row=1, col=1)
    fig.update_yaxes(title_text="回撤 (%)", row=1, col=2, secondary_y=False)
    fig.update_yaxes(title_text="累计收益", row=1, col=2, secondary_y=True)
    fig.update_xaxes(title_text="日收益 (%)", row=2, col=1)
    fig.update_yaxes(title_text="频数", row=2, col=1)
    fig.update_yaxes(title_text="年份", row=2, col=2)
    
    if to_html:
        return fig.to_html(include_plotlyjs=include_plotlyjs, full_html=False)
    return fig


def plot_colored_table(
    df: pd.DataFrame,
    title: str = "策略绩效统计",
    template: str = "plotly_dark",
    float_fmt: str = ".2f",
    to_html: bool = False,
    include_plotlyjs: bool = True,
    **kwargs
) -> Union[go.Figure, str]:
    """绘制带有列独立热力图颜色的表格
    
    :param df: 统计数据 DataFrame
    :param title: 图表标题
    :param template: plotly 模板
    :param float_fmt: 浮点数格式化精度
    :param to_html: 是否返回 HTML
    :param include_plotlyjs: 是否包含 plotly.js
    :param kwargs:
        - good_high_columns: list, 指定哪些列是数值越大越好
        - row_height: int, 行高，默认 30
        - border_color: str, 边框颜色，默认 'darkgrey'
        - header_bgcolor: str, 表头背景色，默认 'grey'
    :return:
    """
    # 准备表头
    headers = df.columns.tolist()
    if df.index.name:
        headers.insert(0, df.index.name)
    else:
        headers.insert(0, "Index")
    
    good_high_columns = kwargs.get("good_high_columns", None)
    row_height = kwargs.get("row_height", 30)
    border_color = kwargs.get("border_color", "darkgrey")
    header_bgcolor = kwargs.get("header_bgcolor", "grey")
    
    # 准备数据和颜色
    cell_values = []
    cell_colors = []
    
    # 处理索引列
    cell_values.append(df.index.tolist())
    # 索引列背景色，使用模板默认背景或透明
    cell_colors.append([header_bgcolor] * len(df))
    
    # 处理数据列
    for col in df.columns:
        series = df[col]
        
        # 格式化数值
        if pd.api.types.is_float_dtype(series):
            # 简单的智能格式化：如果列名包含%，则转为百分比格式
            if "%" in str(col) or "率" in str(col) or "比" in str(col):
                formatted_vals = series.apply(lambda x: f"{x:.2%}")
            else:
                formatted_vals = series.apply(lambda x: f"{x:{float_fmt}}")
            cell_values.append(formatted_vals)
        else:
            cell_values.append(series.tolist())
            
        # 计算颜色
        if pd.api.types.is_numeric_dtype(series):
            # 判断指标方向：默认越大越好（红），含有"回撤"、"风险"则越小越好（红）
            # 注意：这里假设使用 RdYlGn 色标，且红色代表"好"（符合A股习惯）
            # RdYlGn: 0=Red, 1=Green
            
            is_good_high = str(col) in good_high_columns if good_high_columns is not None else True
            
            min_val, max_val = series.min(), series.max()
            if min_val == max_val:
                colors = ['rgba(0,0,0,0)'] * len(series)
            else:
                norm = (series - min_val) / (max_val - min_val)
                
                # RdYlGn: 0=Red, 0.5=Yellow, 1=Green
                # 我们希望：好=Red(0), 坏=Green(1)
                
                if is_good_high:
                    # 越大越好：Max -> Red(0). input = 1 - norm
                    sample_vals = 1 - norm
                else:
                    # 越小越好：Min -> Red(0). input = norm
                    sample_vals = norm
                
                colors = px.colors.sample_colorscale("RdYlGn", sample_vals)
            
            cell_colors.append(colors)
        else:
            cell_colors.append(['rgba(0,0,0,0)'] * len(series))

    fig = go.Figure(data=[go.Table(
        header=dict(
            values=headers,
            fill_color=header_bgcolor,
            align='center',
            font=dict(color='white', size=12),
            height=row_height,
            line=dict(color=border_color, width=1)
        ),
        cells=dict(
            values=cell_values,
            fill_color=cell_colors,
            align='center',
            font=dict(color='white', size=12),
            height=row_height,
            line=dict(color=border_color, width=1)
        )
    )])
    
    fig.update_layout(
        title=title,
        template=template,
        margin=dict(l=10, r=10, t=40, b=10)
    )
    
    if to_html:
        return fig.to_html(include_plotlyjs=include_plotlyjs, full_html=False)
    return fig


def plot_long_short_comparison(
    dailys_pivot: pd.DataFrame,
    stats_df: pd.DataFrame,
    title: str = "多空收益对比",
    template: str = "plotly",
    to_html: bool = False,
    include_plotlyjs: bool = True
) -> Union[go.Figure, str]:
    """绘制多空收益对比图（累计收益曲线 + 绩效指标对比表）
    
    :param dailys_pivot: 透视表格式的日收益数据，index为日期，columns为策略名，values为收益率
    :param stats_df: 绩效指标对比表，每行代表一个策略的指标
    :param title: 图表标题
    :param template: plotly 模板名称
    :param to_html: 是否转换为 HTML
    :param include_plotlyjs: 转换 HTML 时是否包含 plotly.js 库
    :return: Figure 对象或 HTML 字符串
    """
    # 创建 2x1 子图（上下布局），指定第二个子图为 table 类型
    fig = make_subplots(
        rows=2, cols=1,
        subplot_titles=("累计收益曲线对比", "绩效指标对比"),
        vertical_spacing=0.12,
        row_heights=[0.55, 0.45],
        specs=[[{"type": "xy"}], [{"type": "table"}]]
    )
    
    # ========== 上图：累计收益曲线 ==========
    df_cumsum = dailys_pivot.cumsum()
    for col in df_cumsum.columns:
        fig.add_trace(
            go.Scatter(
                x=df_cumsum.index,
                y=df_cumsum[col],
                name=col,
                mode='lines',
            ),
            row=1, col=1
        )
    
    # 添加年度分隔线
    years = df_cumsum.index.year.unique()
    for year in years:
        first_date = df_cumsum[df_cumsum.index.year == year].index.min()
        fig.add_vline(x=first_date, line_dash="dash", line_color="gray", opacity=0.5, row=1, col=1)
    
    # ========== 下图：使用 plot_colored_table 绘制绩效对比表 ==========
    # 选择关键指标列
    key_cols = [
        "策略名称", "年化", "夏普", "卡玛", "最大回撤", 
        "年化波动率", "交易胜率", "单笔收益", "持仓K线数", "多头占比", "空头占比"
    ]
    
    # 过滤存在的列
    available_cols = [col for col in key_cols if col in stats_df.columns]
    table_df = stats_df[available_cols].copy()
    
    # 设置策略名称为索引（如果存在）
    if "策略名称" in table_df.columns:
        table_df = table_df.set_index("策略名称")
    
    # 调用 plot_colored_table 生成表格图表
    table_fig = plot_colored_table(
        table_df,
        title="",
        template=template,
        float_fmt=".2f",
        to_html=False,
        good_high_columns=["年化", "夏普", "卡玛", "交易胜率", "单笔收益"]
    )
    
    # 将表格的 trace 添加到主图中
    for trace in table_fig.data:
        fig.add_trace(trace, row=2, col=1)
    
    # ========== 更新布局 ==========
    fig.update_layout(
        title=title,
        template=template,
        height=900,
        margin=dict(l=20, r=20, b=20, t=60),
        hovermode="x unified",
        showlegend=True,
        legend=dict(x=0.01, y=0.99, bgcolor='rgba(0,0,0,0)')
    )
    
    # 更新坐标轴标签
    fig.update_yaxes(title_text="累计收益", row=1, col=1)
    fig.update_xaxes(title_text="", row=1, col=1)
    
    if to_html:
        return fig.to_html(include_plotlyjs=include_plotlyjs, full_html=False)
    return fig



import os
from loguru import logger
from typing import Tuple, Optional


def check_rs_czsc() -> Tuple[bool, Optional[str]]:
    """
    检查 rs_czsc 库是否正确安装

    Returns:
        Tuple[bool, Optional[str]]: (是否安装成功, 版本号或错误信息)
    """
    try:
        import rs_czsc
        # 尝试获取版本信息
        version = getattr(rs_czsc, '__version__', 'unknown')
        return True, version
    except ImportError as e:
        return False, f"ImportError: {str(e)}"
    except Exception as e:
        return False, f"Error: {str(e)}"

installed, rs_czsc_version = check_rs_czsc()

if os.getenv('CZSC_USE_PYTHON', False) or not installed:
    logger.info("使用 python 版本对象")
    from czsc.py import (
        # 枚举类型
        Operate, Freq, Mark, Direction,
        # 核心分析类
        CZSC,
        # 分析函数
        remove_include, check_bi, check_fx, check_fxs,
        # K线生成器
        BarGenerator, freq_end_time, is_trading_time, format_standard_kline,
        # 数据对象
        RawBar, NewBar, FX, BI, FakeBI, ZS, Signal, Event, Position,
        # 回测
        WeightBacktest
    )
else:
    logger.info("使用 rust 版本对象")
    # 导入已经用 rust 复现的函数
    from rs_czsc import (
        # 枚举类型
        Operate, Freq, Mark, Direction,
        # 核心分析类
        CZSC,
        # K线生成器
        BarGenerator, format_standard_kline,
        # 数据对象
        RawBar, NewBar, FX, BI, FakeBI, ZS, Signal, Event, Position,
        # 回测
        WeightBacktest
    )
    # 从 py 模块导入未复现的函数
    from czsc.py import (
        # 分析函数（rust版本未实现）
        remove_include, check_bi, check_fx, check_fxs,
        # K线生成器函数（rust版本未实现）
        freq_end_time, is_trading_time
    )
    

__all__ = [
    # 枚举类型
    "Operate", "Freq", "Mark", "Direction",
    # 核心分析类
    "CZSC",
    # 分析函数
    "remove_include", "check_bi", "check_fx", "check_fxs",
    # K线生成器
    "BarGenerator", "freq_end_time", "is_trading_time", "format_standard_kline",
    # 数据对象
    "RawBar", "NewBar", "FX", "BI", "FakeBI", "ZS", "Signal", "Event", "Position",
    # 回测
    "WeightBacktest",
    # 工具函数
    "check_rs_czsc"
]    



"""
输入标准的策略持仓数据，分析换手率、持仓权重的分布

标准的策略持仓数据含有 dt、symbol、weight 三列，样例输入如下：
         dt symbol    weight
0 2010-01-01   AAPL -0.255969
1 2010-01-02   AAPL  0.634864
2 2010-01-03   AAPL -1.000000
3 2010-01-04   AAPL -0.516123
4 2010-01-05   AAPL -0.411114
"""

import pandas as pd
import plotly.graph_objects as go
from plotly.subplots import make_subplots
import plotly.express as px
import numpy as np
from scipy import stats
from typing import Dict, List


def calculate_weight_stats(dfw: pd.DataFrame) -> pd.DataFrame:
    """
    计算权重统计数据

    Args:
        dfw: 包含dt、symbol、weight列的DataFrame

    Returns:
        包含权重统计的DataFrame
    """
    # 计算各类统计量
    long_total = dfw[dfw['weight'] > 0].groupby('dt')['weight'].sum()
    short_total = dfw[dfw['weight'] < 0].groupby('dt')['weight'].sum().abs()  # 转为正值
    abs_total = dfw.groupby('dt')['weight'].apply(lambda x: x.abs().sum())
    net_total = dfw.groupby('dt')['weight'].sum()
    position_count = dfw[dfw['weight'] != 0].groupby('dt').size()

    # 合并所有统计量
    weight_stats = pd.DataFrame({
        'dt': dfw['dt'].unique(),
        'long_total': long_total,
        'short_total': short_total,
        'abs_total': abs_total,
        'net_total': net_total,
        'position_count': position_count
    }).fillna(0)
    return weight_stats


def plot_weight_time_series(dfw: pd.DataFrame,
                           title: str = "策略持仓权重分布分析",
                           height: int = 800,
                           show_position_count: bool = True) -> go.Figure:
    """
    绘制权重暴露曲线时序图

    Args:
        dfw: 包含dt、symbol、weight列的DataFrame
        title: 图表标题
        height: 图表高度
        show_position_count: 是否显示持仓数量

    Returns:
        Plotly图表对象
    """
    # 计算权重统计数据
    weight_stats = calculate_weight_stats(dfw)

    if show_position_count:
        # 创建双子图布局
        fig = make_subplots(
            rows=2, cols=1,
            subplot_titles=('持仓权重分布', '净仓位和持仓数量'),
            vertical_spacing=0.12,
            specs=[[{"secondary_y": False}], [{"secondary_y": True}]]
        )
    else:
        # 单图布局
        fig = make_subplots(
            rows=1, cols=1,
            subplot_titles=('持仓权重分布',)
        )

    # 添加多头累计、空头累计、绝对仓位曲线
    fig.add_trace(
        go.Scatter(
            x=weight_stats['dt'],
            y=weight_stats['long_total'],
            mode='lines',
            name='多头累计',
            line=dict(color='green', width=2),
            hovertemplate='<b>%{fullData.name}</b><br>时间: %{x}<br>权重: %{y:.3f}<extra></extra>'
        ),
        row=1, col=1
    )

    fig.add_trace(
        go.Scatter(
            x=weight_stats['dt'],
            y=weight_stats['short_total'],
            mode='lines',
            name='空头累计',
            line=dict(color='red', width=2),
            hovertemplate='<b>%{fullData.name}</b><br>时间: %{x}<br>权重: %{y:.3f}<extra></extra>'
        ),
        row=1, col=1
    )

    fig.add_trace(
        go.Scatter(
            x=weight_stats['dt'],
            y=weight_stats['abs_total'],
            mode='lines',
            name='绝对仓位',
            line=dict(color='blue', width=2),
            hovertemplate='<b>%{fullData.name}</b><br>时间: %{x}<br>权重: %{y:.3f}<extra></extra>'
        ),
        row=1, col=1
    )

    # 添加净仓位曲线
    if show_position_count:
        fig.add_trace(
            go.Scatter(
                x=weight_stats['dt'],
                y=weight_stats['net_total'],
                mode='lines',
                name='净仓位',
                line=dict(color='purple', width=2),
                hovertemplate='<b>%{fullData.name}</b><br>时间: %{x}<br>净仓位: %{y:.3f}<extra></extra>'
            ),
            row=2, col=1
        )

        # 添加持仓数量柱状图（使用次坐标轴）
        fig.add_trace(
            go.Bar(
                x=weight_stats['dt'],
                y=weight_stats['position_count'],
                name='持仓数量',
                marker_color='orange',
                opacity=0.7,
                hovertemplate='<b>%{fullData.name}</b><br>时间: %{x}<br>数量: %{y}<extra></extra>'
            ),
            row=2, col=1,
            secondary_y=True
        )

    # 更新布局
    fig.update_layout(
        title={
            'text': title,
            'x': 0.5,
            'xanchor': 'center',
            'font': {'size': 20}
        },
        height=height,
        showlegend=True,
        legend=dict(
            orientation="h",
            yanchor="bottom",
            y=1.02,
            xanchor="right",
            x=1
        ),
        hovermode='x unified',
        plot_bgcolor='white',
        paper_bgcolor='white'
    )

    # 更新x轴
    fig.update_xaxes(
        title_text="时间",
        title_font=dict(size=14),
        tickfont=dict(size=12)
    )

    # 更新y轴
    fig.update_yaxes(
        title_text="权重",
        title_font=dict(size=14),
        tickfont=dict(size=12),
        row=1, col=1
    )

    if show_position_count:
        fig.update_yaxes(
            title_text="净仓位",
            title_font=dict(size=14),
            tickfont=dict(size=12),
            row=2, col=1
        )

        fig.update_yaxes(
            title_text="持仓数量",
            title_font=dict(size=14),
            tickfont=dict(size=12),
            row=2, col=1,
            secondary_y=True
        )

    return fig


def plot_weight_histogram_kde(dfw: pd.DataFrame,
                             title: str = "仓位分布直方图与核密度估计",
                             height: int = 800,
                             width: int = 900) -> go.Figure:
    """
    绘制仓位分布直方图与核密度估计

    Args:
        dfw: 包含dt、symbol、weight列的DataFrame
        title: 图表标题
        height: 图表高度
        width: 图表宽度

    Returns:
        Plotly图表对象
    """
    # 计算权重统计数据
    weight_stats = calculate_weight_stats(dfw)
    # 创建子图布局
    fig_hist = make_subplots(
        rows=2, cols=2,
        subplot_titles=('多头仓位分布', '空头仓位分布', '净仓位分布', '绝对仓位分布'),
        vertical_spacing=0.15,
        horizontal_spacing=0.1
    )

    # 准备仓位数据
    position_data = {
        '多头仓位': weight_stats['long_total'],
        '空头仓位': weight_stats['short_total'],  # 已经是正值
        '净仓位': weight_stats['net_total'],
        '绝对仓位': weight_stats['abs_total']
    }

    positions = ['多头仓位', '空头仓位', '净仓位', '绝对仓位']
    position_indices = [(1,1), (1,2), (2,1), (2,2)]
    colors = ['#2E8B57', '#DC143C', '#4682B4', '#FF8C00']

    for i, (position_type, (row, col)) in enumerate(zip(positions, position_indices)):
        data = position_data[position_type]
        color = colors[i]

        # 计算核密度估计
        kde = stats.gaussian_kde(data)
        x_range = np.linspace(data.min(), data.max(), 200)
        kde_values = kde(x_range)

        # 添加直方图
        fig_hist.add_trace(
            go.Histogram(
                x=data,
                nbinsx=50,
                name=f'{position_type}_直方图',
                marker_color=color,
                opacity=0.7,
                histnorm='probability density',  # 使用概率密度
                showlegend=False
            ),
            row=row, col=col
        )

        # 添加核密度估计曲线
        fig_hist.add_trace(
            go.Scatter(
                x=x_range,
                y=kde_values,
                mode='lines',
                name=f'{position_type}_KDE',
                line=dict(color='black', width=2),
                showlegend=False
            ),
            row=row, col=col
        )

        # 添加统计信息文本
        mean_val = data.mean()
        median_val = data.median()
        std_val = data.std()

        fig_hist.add_annotation(
            x=data.max() * 0.7,
            y=kde_values.max() * 0.8,
            text=f'均值: {mean_val:.3f}<br>中位数: {median_val:.3f}<br>标准差: {std_val:.3f}',
            showarrow=False,
            font=dict(size=10),
            bgcolor="rgba(255,255,255,0.8)",
            row=row, col=col
        )

    # 更新布局
    fig_hist.update_layout(
        title={
            'text': title,
            'x': 0.5,
            'xanchor': 'center',
            'font': {'size': 18}
        },
        height=height,
        width=width,
        showlegend=False,
        plot_bgcolor='white',
        paper_bgcolor='white'
    )

    # 更新所有子图的坐标轴
    for i in range(1, 3):
        for j in range(1, 3):
            fig_hist.update_xaxes(
                title_text="仓位值" if i == 2 else "",
                linecolor='black',
                linewidth=1,
                tickfont=dict(size=10),
                row=i, col=j
            )
            fig_hist.update_yaxes(
                title_text="密度" if j == 1 else "",
                linecolor='black',
                linewidth=1,
                tickfont=dict(size=10),
                row=i, col=j
            )

    return fig_hist


def plot_weight_cdf(dfw: pd.DataFrame,
                   title: str = "仓位分布累积分布函数(CDF)对比",
                   height: int = 600,
                   width: int = 800,
                   show_percentiles: bool = True) -> go.Figure:
    """
    绘制仓位分布累积分布函数图

    Args:
        dfw: 包含dt、symbol、weight列的DataFrame
        title: 图表标题
        height: 图表高度
        width: 图表宽度
        show_percentiles: 是否显示分位数参考线

    Returns:
        Plotly图表对象
    """
    # 计算权重统计数据
    weight_stats = calculate_weight_stats(dfw)
    # 创建累积分布函数图
    fig_cdf = go.Figure()

    # 准备仓位数据
    position_data = {
        '多头仓位': weight_stats['long_total'],
        '空头仓位': weight_stats['short_total'],  # 已经是正值
        '净仓位': weight_stats['net_total'],
        '绝对仓位': weight_stats['abs_total']
    }

    colors = ['#2E8B57', '#DC143C', '#4682B4', '#FF8C00']

    for i, (position_type, data) in enumerate(position_data.items()):
        color = colors[i]

        # 计算累积分布函数
        sorted_data = np.sort(data)
        n = len(sorted_data)
        cdf_y = np.arange(1, n + 1) / n

        # 添加CDF曲线
        fig_cdf.add_trace(go.Scatter(
            x=sorted_data,
            y=cdf_y,
            mode='lines',
            name=position_type,
            line=dict(color=color, width=2.5),
            hovertemplate='<b>%{fullData.name}</b><br>仓位: %{x:.3f}<br>累积概率: %{y:.3f}<extra></extra>'
        ))

    # 添加参考线
    if show_percentiles:
        fig_cdf.add_hline(y=0.25, line_dash="dash", line_color="gray", annotation_text="25%分位数")
        fig_cdf.add_hline(y=0.5, line_dash="dash", line_color="gray", annotation_text="中位数")
        fig_cdf.add_hline(y=0.75, line_dash="dash", line_color="gray", annotation_text="75%分位数")

    # 更新布局
    fig_cdf.update_layout(
        title={
            'text': title,
            'x': 0.5,
            'xanchor': 'center',
            'font': {'size': 18}
        },
        xaxis_title="仓位值",
        yaxis_title="累积概率",
        height=height,
        width=width,
        showlegend=True,
        legend=dict(
            x=0.02,
            y=0.98,
            bgcolor="rgba(255,255,255,0.8)",
            bordercolor="black",
            borderwidth=1
        ),
        plot_bgcolor='white',
        paper_bgcolor='white'
    )

    # 更新坐标轴
    fig_cdf.update_xaxes(
        linecolor='black',
        linewidth=2,
        tickfont=dict(size=12),
        showgrid=True,
        gridwidth=1,
        gridcolor='lightgray'
    )

    fig_cdf.update_yaxes(
        linecolor='black',
        linewidth=2,
        tickfont=dict(size=12),
        showgrid=True,
        gridwidth=1,
        gridcolor='lightgray',
        range=[0, 1]
    )

    return fig_cdf


def plot_absolute_position_analysis(dfw: pd.DataFrame,
                                  title: str = "绝对仓位详细分析图表",
                                  height: int = 900,
                                  width: int = 1000,
                                  ma_windows: List[int] = [5, 20, 60],
                                  volatility_window: int = 20) -> go.Figure:
    """
    绘制绝对仓位详细分析图

    Args:
        dfw: 包含dt、symbol、weight列的DataFrame
        title: 图表标题
        height: 图表高度
        width: 图表宽度
        ma_windows: 移动平均线窗口列表
        volatility_window: 波动率计算窗口

    Returns:
        Plotly图表对象
    """
    # 计算权重统计数据
    weight_stats = calculate_weight_stats(dfw)
    # 创建子图布局
    fig_abs = make_subplots(
        rows=3, cols=1,
        subplot_titles=('绝对仓位时序曲线', '滚动统计指标', '仓位分布统计'),
        vertical_spacing=0.08,
        row_heights=[0.5, 0.3, 0.2]
    )

    # 1. 绝对仓位时序曲线（主图）
    fig_abs.add_trace(
        go.Scatter(
            x=weight_stats['dt'],
            y=weight_stats['abs_total'],
            mode='lines',
            name='绝对仓位',
            line=dict(color='blue', width=2),
            hovertemplate='<b>绝对仓位</b><br>时间: %{x}<br>仓位: %{y:.3f}<extra></extra>'
        ),
        row=1, col=1
    )

    # 添加移动平均线
    colors_ma = ['orange', 'green', 'red']
    for window, color in zip(ma_windows, colors_ma):
        if len(weight_stats) >= window:
            ma_values = weight_stats['abs_total'].rolling(window=window).mean()
            fig_abs.add_trace(
                go.Scatter(
                    x=weight_stats['dt'],
                    y=ma_values,
                    mode='lines',
                    name=f'MA{window}',
                    line=dict(color=color, width=1.5, dash='dot'),
                    hovertemplate=f'<b>MA{window}</b><br>时间: %{{x}}<br>平均值: %{{y:.3f}}<extra></extra>'
                ),
                row=1, col=1
            )

    # 添加统计参考线
    mean_abs = weight_stats['abs_total'].mean()
    median_abs = weight_stats['abs_total'].median()

    # 添加水平参考线
    fig_abs.add_shape(
        type="line",
        x0=weight_stats['dt'].min(),
        y0=mean_abs,
        x1=weight_stats['dt'].max(),
        y1=mean_abs,
        line=dict(color="red", width=2, dash="dash"),
        row=1, col=1
    )

    # 计算注释位置（使用90%时间位置）
    annotation_x = weight_stats['dt'].min() + (weight_stats['dt'].max() - weight_stats['dt'].min()) * 0.9
    fig_abs.add_annotation(
        x=annotation_x,
        y=mean_abs,
        text=f"均值: {mean_abs:.3f}",
        showarrow=False,
        font=dict(color="red"),
        row=1, col=1
    )

    fig_abs.add_shape(
        type="line",
        x0=weight_stats['dt'].min(),
        y0=median_abs,
        x1=weight_stats['dt'].max(),
        y1=median_abs,
        line=dict(color="green", width=2, dash="dash"),
        row=1, col=1
    )

    fig_abs.add_annotation(
        x=annotation_x,
        y=median_abs,
        text=f"中位数: {median_abs:.3f}",
        showarrow=False,
        font=dict(color="green"),
        row=1, col=1
    )

    # 2. 滚动统计指标（第二幅图）
    # 计算滚动标准差和波动率
    rolling_std = weight_stats['abs_total'].rolling(window=volatility_window).std()
    rolling_volatility = (rolling_std / weight_stats['abs_total'].rolling(window=volatility_window).mean()) * 100

    fig_abs.add_trace(
        go.Scatter(
            x=weight_stats['dt'],
            y=rolling_volatility,
            mode='lines',
            name=f'{volatility_window}日滚动波动率(%)',
            line=dict(color='purple', width=2),
            hovertemplate='<b>滚动波动率</b><br>时间: %{x}<br>波动率: %{y:.2f}%<extra></extra>'
        ),
        row=2, col=1
    )

    # 3. 仓位分布直方图（第三幅图）
    fig_abs.add_trace(
        go.Histogram(
            x=weight_stats['abs_total'],
            nbinsx=50,
            name='绝对仓位分布',
            marker_color='lightblue',
            opacity=0.7,
            showlegend=False
        ),
        row=3, col=1
    )

    # 更新布局
    fig_abs.update_layout(
        title={
            'text': title,
            'x': 0.5,
            'xanchor': 'center',
            'font': {'size': 20}
        },
        height=height,
        width=width,
        showlegend=True,
        legend=dict(
            x=0.02,
            y=0.98,
            bgcolor="rgba(255,255,255,0.8)",
            bordercolor="black",
            borderwidth=1
        ),
        plot_bgcolor='white',
        paper_bgcolor='white',
        hovermode='x unified'
    )

    # 更新坐标轴
    # 主图坐标轴
    fig_abs.update_xaxes(
        showgrid=True,
        gridwidth=1,
        gridcolor='lightgray',
        linecolor='black',
        linewidth=2,
        tickfont=dict(size=12),
        row=1, col=1
    )

    fig_abs.update_yaxes(
        title_text="绝对仓位大小",
        showgrid=True,
        gridwidth=1,
        gridcolor='lightgray',
        linecolor='black',
        linewidth=2,
        tickfont=dict(size=12),
        row=1, col=1
    )

    # 波动率图坐标轴
    fig_abs.update_xaxes(
        showgrid=True,
        gridwidth=1,
        gridcolor='lightgray',
        linecolor='black',
        linewidth=2,
        tickfont=dict(size=12),
        row=2, col=1
    )

    fig_abs.update_yaxes(
        title_text="波动率 (%)",
        showgrid=True,
        gridwidth=1,
        gridcolor='lightgray',
        linecolor='black',
        linewidth=2,
        tickfont=dict(size=12),
        row=2, col=1
    )

    # 分布图坐标轴
    fig_abs.update_xaxes(
        title_text="绝对仓位值",
        showgrid=True,
        gridwidth=1,
        gridcolor='lightgray',
        linecolor='black',
        linewidth=2,
        tickfont=dict(size=12),
        row=3, col=1
    )

    fig_abs.update_yaxes(
        title_text="频数",
        showgrid=True,
        gridwidth=1,
        gridcolor='lightgray',
        linecolor='black',
        linewidth=2,
        tickfont=dict(size=12),
        row=3, col=1
    )

    return fig_abs


if __name__ == "__main__":
    import czsc

    # 生成模拟数据
    dfw = czsc.mock.generate_klines_with_weights()
    dfw = dfw[['dt', 'symbol', 'weight']].copy()

    # 或者单独使用某个函数
    # fig_time = plot_weight_time_series(dfw)
    # fig_hist = plot_weight_histogram_kde(dfw)
    # fig_cdf = plot_weight_cdf(dfw)
    # fig_abs = plot_absolute_position_analysis(dfw)
    # fig_time.show()


"""
工具类组件

包含代码编辑器等辅助功能
"""
import sys
import streamlit as st


def streamlit_run(file, port=8501, host="localhost"):
    """
    直接启动 Streamlit 应用，添加运行时检查
    """
    from loguru import logger
    from streamlit.web import cli as stcli

    logger.info("streamlit_run -- 启动 Streamlit 应用...")
    logger.info(f"UI 文件路径: {file}")

    # 使用 streamlit.web.cli.main
    sys.argv = [
        "streamlit", 
        "run", 
        str(file),
        "--server.port", str(port),
        "--server.address", host,
        "--server.headless", "true",
        "--server.maxUploadSize", "2048"
    ]

    stcli.main()



import os
import time
from pathlib import Path
import czsc
import ccxt
import pandas as pd
import loguru
from tenacity import retry, stop_after_attempt, wait_fixed


def __get_exchange(exchange="币安期货"):
    """获取交易所连接"""
    if exchange == "币安期货":
        e = ccxt.binanceusdm()
    elif exchange == "币安现货":
        e = ccxt.binance()
    else:
        raise ValueError(f"不支持的交易类型: {exchange}")

    use_proxy = os.getenv("USE_PROXY", "0") == "1"
    proxies = {
        "http": os.getenv("HTTP_PROXY", "http://127.0.0.1:10808"),
        "https": os.getenv("HTTPS_PROXY", "http://127.0.0.1:10808"),
    }
    if use_proxy:
        # v2rayN的http代理设置样例
        # proxies = {"http": "http://127.0.0.1:10809", "https": "http://127.0.0.1:10809"}
        e.proxies = proxies
    return e


def get_symbols(exchange="币安期货", **kwargs):
    """获取交易品种列表

    df = get_symbols(kind="币安期货")
    df = get_symbols(kind="币安现货")
    """
    exchange = __get_exchange(exchange=exchange)

    markets = exchange.load_markets()
    df = pd.DataFrame(
        [
            {
                "ccxt_symbol": symbol,
                "base": market["base"],
                "quote": market["quote"],
                "price_size": market["precision"]["price"],
            }
            for symbol, market in markets.items()
        ]
    )
    df["symbol"] = df["base"] + df["quote"]
    return df


@czsc.disk_cache(ttl=-1)
def __binance_fetch_ohlcv_once(exchange, symbol, since, interval, logger=loguru.logger):
    """获取币安交易所的K线数据"""
    all_klines = []
    params = {
        "symbol": symbol.replace("/", ""),  # 转换为币安格式
        "interval": interval,
        "startTime": since,
        "limit": 1000,
    }
    if exchange.name == "Binance USDⓈ-M":
        raw_klines = exchange.fapiPublicGetKlines(params)
        logger.info(f"{exchange.name} - 获取 {params} K线数据量为 {len(raw_klines)}")
    elif exchange.name == "Binance":
        raw_klines = exchange.publicGetKlines(params)
        logger.info(f"{exchange.name} - 获取 {params} K线数据量为 {len(raw_klines)}")
    else:
        raise ValueError(f"不支持的交易所: {exchange.name}")

    if not raw_klines:
        logger.warning(f"获取 {params} K线数据量为 0 ")
        return [], since

    # 转换数据结构
    for kline in raw_klines:
        processed_kline = {
            "start_time": int(kline[0]),  # timestamp
            "open": float(kline[1]),  # open
            "high": float(kline[2]),  # high
            "low": float(kline[3]),  # low
            "close": float(kline[4]),  # close
            "vol": float(kline[5]),  # volume (base)
            "dt": int(kline[6]) + 1,  # close time
            "amount": float(kline[7]),  # quote volume
            "trades": int(kline[8]),  # number of trades
            "taker_buy_volume": float(kline[9]),  # taker buy base volume
            "taker_buy_quote_volume": float(kline[10]),  # taker buy quote volume
        }
        all_klines.append(processed_kline)

    logger.info(f"获取 {params} K线数据量为 {len(all_klines)}")
    since = int(raw_klines[-1][0]) + 1
    return all_klines, since


# @retry(stop=stop_after_attempt(3), wait=wait_fixed(3))
def __binance_fetch_ohlcv(exchange, symbol, sdt, edt, interval):
    """获取币安交易所的K线数据"""
    # 转换时间戳
    since = int(pd.to_datetime(sdt).timestamp() * 1000)
    until = int(pd.to_datetime(edt).timestamp() * 1000)

    all_klines = []
    while since < until:
        _klines, _since = __binance_fetch_ohlcv_once(exchange, symbol, since, interval)
        if not _klines:
            break
        all_klines.extend(_klines)
        since = _since
        time.sleep(exchange.rateLimit / 1000)

    if not all_klines:
        return pd.DataFrame()

    df = pd.DataFrame(all_klines)
    # dt 是K线结束时间；时区转换：UTC -> Asia/Shanghai
    df["dt"] = pd.to_datetime(df["dt"], unit="ms") + pd.Timedelta(hours=8)
    df = df[["dt", "open", "high", "low", "close", "vol", "amount"]]
    return df


def __exchange_fetch_ohlcv(exchange, symbol, sdt, edt, interval):
    """获取交易所的K线数据"""
    # 转换时间戳
    since = int(pd.to_datetime(sdt).timestamp() * 1000)
    until = int(pd.to_datetime(edt).timestamp() * 1000)

    all_klines = []
    while since < until:
        klines = exchange.fetch_ohlcv(symbol=symbol, timeframe=interval, since=since, limit=1000)
        if not klines:
            break

        all_klines.extend(klines)
        since = klines[-1][0] + 1  # 更新获取时间

        # 添加延时避免超过API限制
        time.sleep(0.5)

    if not all_klines:
        return pd.DataFrame()

    df = pd.DataFrame(all_klines, columns=["dt", "open", "high", "low", "close", "vol"])
    df["amount"] = df["vol"] * df["close"]

    # dt 转成K线结束时间
    if interval == "4h":
        df["dt"] = pd.to_datetime(df["dt"], unit="ms") + pd.Timedelta(hours=4)

    elif interval == "2h":
        df["dt"] = pd.to_datetime(df["dt"], unit="ms") + pd.Timedelta(hours=2)

    elif interval == "1h":
        df["dt"] = pd.to_datetime(df["dt"], unit="ms") + pd.Timedelta(hours=1)

    elif interval == "30m":
        df["dt"] = pd.to_datetime(df["dt"], unit="ms") + pd.Timedelta(minutes=30)

    elif interval == "15m":
        df["dt"] = pd.to_datetime(df["dt"], unit="ms") + pd.Timedelta(minutes=15)

    elif interval == "5m":
        df["dt"] = pd.to_datetime(df["dt"], unit="ms") + pd.Timedelta(minutes=5)

    elif interval == "1m":
        df["dt"] = pd.to_datetime(df["dt"], unit="ms") + pd.Timedelta(minutes=1)

    # dt 是K线结束时间；时区转换：UTC -> Asia/Shanghai
    df["dt"] = pd.to_datetime(df["dt"], unit="ms") + pd.Timedelta(hours=8)
    return df


def get_raw_bars(symbol="BTCUSDT", period="4h", sdt="20240101", edt="20240308", **kwargs):
    """获取指定交易对的K线数据

    Args:
        symbol: 交易对，如 "BTCUSDT"
        period: 时间周期，支持 1m/5m/15m/1h/2h/4h/6h/8h/12h/1d
        sdt: 开始时间
        edt: 结束时间

    Returns:
        pandas.DataFrame: 包含K线数据的DataFrame
    """
    logger = kwargs.get("logger", loguru.logger)
    exchange = kwargs.get("exchange", "币安期货")
    e = __get_exchange(exchange=exchange)

    # 转换时间格式
    timeframes = {
        "1m": "1m",
        "5m": "5m",
        "15m": "15m",
        "30m": "30m",
        "1h": "1h",
        "2h": "2h",
        "4h": "4h",
        "6h": "6h",
        "8h": "8h",
        "12h": "12h",
        "1d": "1d",
    }
    if period not in timeframes:
        raise ValueError(f"不支持的时间周期: {period}")

    if exchange in ["币安期货", "币安现货"]:
        df = __binance_fetch_ohlcv(e, symbol, sdt, edt, timeframes[period])
    else:
        raise ValueError(f"不支持的交易所: {exchange}")
        # df = __exchange_fetch_ohlcv(e, symbol, sdt, edt, timeframes[period])

    df = df.sort_values("dt").reset_index(drop=True)
    df = df.drop_duplicates("dt", keep="last")
    df["symbol"] = symbol
    df = df[(df["dt"] <= pd.to_datetime(edt)) & (df["dt"] >= pd.to_datetime(sdt))]

    # 过滤未完成的K线
    if df["dt"].max() > pd.Timestamp.now():
        df = df.iloc[:-1]
    logger.info(f"get_raw_bars::获取 {symbol} {period} K线数据，时间段：{sdt} - {edt}，K线数量：{len(df)}")
    return df


def get_latest_klines(symbol, period, sdt=None, **kwargs):
    """获取最新的K线数据

    :param symbol: 交易对
    :param period: K线周期; 1m/5m/15m/30m/1h/2h/4h/1d
    :param sdt: 开始时间
    :param kwargs:

        - logger: loguru.logger, 默认为 loguru.logger
        - cache_path: str, 缓存路径，默认为 czsc.home_path
        - proxies: dict, 代理设置，默认为 None
        - kind: str, 交易类型，默认为 "币安期货"

    """
    logger = kwargs.get("logger", loguru.logger)
    cache_path = Path(kwargs.get("cache_path", czsc.home_path))
    cache_path.mkdir(exist_ok=True, parents=True)

    sdt = pd.to_datetime(sdt) if sdt else pd.to_datetime("20170101")
    edt = pd.Timestamp.now() + pd.Timedelta(days=2)
    file_cache = Path(f"{cache_path}/klines_{symbol}_{period}_{sdt.strftime('%Y%m%d')}.feather")

    if file_cache.exists():
        df = pd.read_feather(file_cache)
        logger.info(f"读取缓存数据：{file_cache}，最新时间：{df['dt'].max()}")
        _sdt = df["dt"].max() - pd.Timedelta(days=1)
    else:
        df = pd.DataFrame()
        _sdt = sdt

    proxies = kwargs.get("proxies")
    exchange = kwargs.get("exchange", "币安期货")

    df1 = get_raw_bars(symbol=symbol, period=period, sdt=_sdt, edt=edt, proxies=proxies, exchange=exchange)
    logger.info(f"获取 {symbol} {period} K线数据，时间段：{_sdt} - {edt}，K线数量：{len(df1)}")

    df2 = pd.concat([df, df1], ignore_index=True)
    df2 = df2.drop_duplicates("dt", keep="last")
    df2 = df2.sort_values("dt").reset_index(drop=True)
    df2.to_feather(file_cache)

    df3 = df2[df2["dt"] > sdt].reset_index(drop=True)
    logger.info(f"获取 {symbol} {period} K线数据，时间段：{df3['dt'].min()} - {df3['dt'].max()}，K线数量：{len(df3)}")
    return df3


def test_fetcher():
    import os

    os.environ["USE_PROXY"] = "1"

    df = get_symbols()
    print(df)

    # df1 = get_raw_bars(symbol="BTCUSDT", period="4h", sdt="2017-08-17 00:00:00", edt="20240817 20:00")
    df1 = get_raw_bars(
        symbol="ADABTC",
        period="8h",
        sdt="2024-08-17 00:00:00",
        edt="20241110 20:00",
        exchange="币安现货",
    )
    # symbols = ["ADABTC", "AVAXBTC", "BNBBTC", "DOGEBTC", "ETHBTC", "LTCBTC", "SOLBTC", "TRXBTC", "XRPBTC"]
    # rows = []
    # for symbol in symbols:
    #     df1 = get_raw_bars(
    #         symbol=symbol,
    #         period="1h",
    #         sdt="2024-08-17 00:00:00",
    #         edt="20241113 20:00",
    #         exchange="币安现货",
    #     )
    #     rows.append(df1)
    #
    # df = get_latest_klines(symbol="BTCUSDT", period="1h", sdt="2021-01-01", exchange="币安现货")



# -*- coding: utf-8 -*-
"""
author: zengbin93
email: zeng_bin8888@163.com
create_dt: 2019/10/29 15:01
"""
# 尝试从 rs_czsc 导入，失败则使用 Python 版本
try:
    from rs_czsc import (
        daily_performance,
        top_drawdowns,
        # python版本：from czsc.traders.weight_backtest import WeightBacktest
        WeightBacktest,
    )
except ImportError:
    # 回退到 Python 版本
    from czsc.core import WeightBacktest
    # daily_performance 和 top_drawdown 在 Python 版本中的位置
    from czsc.utils import daily_performance, top_drawdowns

from czsc import envs
from czsc import fsa
from czsc import utils
from czsc import traders
from czsc import sensors
from czsc import aphorism
from czsc import svc
from czsc import mock
from czsc.traders import rwc
from czsc.traders import cwc
from czsc.core import CZSC, Freq, Operate, Direction, Signal, Event, RawBar, NewBar, Position, ZS, format_standard_kline
from czsc.strategies import CzscStrategyBase, CzscJsonStrategy
from czsc.sensors import holds_concepts_effect, CTAResearch, EventMatchSensor
from czsc.sensors.feature import FixedNumberSelector
from czsc.utils import ta
from czsc.traders import (
    CzscTrader,
    CzscSignals,
    generate_czsc_signals,
    check_signals_acc,
    get_unique_signals,
    PairsPerformance,
    combine_holds_and_pairs,
    combine_dates_and_pairs,
    DummyBacktest,
    SignalsParser,
    get_signals_config,
    get_signals_freqs,
    stoploss_by_direction,
    get_ensemble_weight,
    OpensOptimize,
    ExitsOptimize,
)

from czsc.traders.rwc import (
    RedisWeightsClient,
    get_strategy_mates,
    get_strategy_names,
    get_heartbeat_time,
    clear_strategy,
    get_strategy_weights,
    get_strategy_latest,
)

from czsc.utils import (
    timeout_decorator,
    mac_address,
    overlap,
    to_arrow,
    # format_standard_kline,
    # BarGenerator,
    # freq_end_time,
    # resample_bars,
    # is_trading_time,
    # get_intraday_times,
    # check_freq_and_market,
    dill_dump,
    dill_load,
    read_json,
    save_json,
    get_sub_elements,
    get_py_namespace,
    code_namespace,
    freqs_sorted,
    x_round,
    import_by_name,
    create_grid_params,
    update_bbars,
    update_tbars,
    update_nxb,
    risk_free_returns,
    resample_to_daily,
    cross_sectional_ranker,
    cross_sectional_ic,
    rolling_daily_performance,
    holds_performance,
    psi,
    home_path,
    DiskCache,
    disk_cache,
    clear_cache,
    clear_expired_cache,
    get_dir_size,
    empty_cache_path,
    print_df_sample,
    index_composition,
    AliyunOSS,
    DataClient,
    set_url_token,
    get_url_token,
    generate_fernet_key,
    fernet_encrypt,
    fernet_decrypt,
)

# 警告信息捕获工具
from czsc.utils.warning_capture import (
    capture_warnings,
    execute_with_warning_capture,
)

# 交易日历工具
from czsc.py.calendar import (
    is_trading_date,
    next_trading_date,
    prev_trading_date,
    get_trading_dates,
)

from czsc.utils.trade import (
    adjust_holding_weights,
)

from czsc.utils.log import (
    log_strategy_info,
)

# streamlit 量化分析组件
from czsc.utils.st_components import (
    show_daily_return,
    show_yearly_stats,
    show_splited_daily,
    show_monthly_return,
    show_correlation,
    show_corr_graph,
    show_sectional_ic,
    show_factor_layering,
    show_weight_backtest,
    show_ts_rolling_corr,
    show_ts_self_corr,
    show_stoploss_by_direction,
    show_cointegration,
    show_out_in_compare,
    show_optuna_study,
    show_drawdowns,
    show_rolling_daily_performance,
    show_event_return,
    show_psi,
    show_holds_backtest,
    show_symbols_corr,
    show_feature_returns,
    show_czsc_trader,
    show_strategies_recent,
    show_factor_value,
    show_code_editor,
    show_classify,
    show_df_describe,
    show_date_effect,
    show_weight_distribution,
    show_normality_check,
    show_outsample_by_dailys,
    show_returns_contribution,
    show_symbols_bench,
    show_quarterly_effect,
    show_cumulative_returns,
    show_cta_periods_classify,
    show_volatility_classify,
    show_portfolio,
    show_turnover_rate,
    show_describe,
    show_event_features,
    show_stats_compare,
    show_symbol_penalty,
)

from czsc.utils.bi_info import (
    calculate_bi_info,
    symbols_bi_infos,
)

from czsc.utils.features import (
    normalize_feature,
    normalize_ts_feature,
    feature_cross_layering,
    find_most_similarity,
)

from czsc.features.utils import (
    is_event_feature,
    rolling_corr,
    rolling_rank,
    rolling_norm,
    rolling_qcut,
    rolling_compare,
    rolling_scale,
    rolling_slope,
    rolling_tanh,
    normalize_corr,
    feature_returns,
    feature_sectional_corr,
)

from czsc.utils.plotly_plot import (
    plot_czsc_chart,
    KlineChart
)

from czsc.utils.kline_quality import check_kline_quality
from czsc.traders import cwc


from czsc.eda import (
    remove_beta_effects,
    vwap,
    twap,
    cross_sectional_strategy,
    judge_factor_direction,
    monotonicity,
    min_max_limit,
    rolling_layers,
    cal_symbols_factor,
    weights_simple_ensemble,
    unify_weights,
    sma_long_bear,
    dif_long_bear,
    tsf_type,
    limit_leverage,
    cal_trade_price,
    mark_cta_periods,
    mark_volatility,
    cal_yearly_days,
    turnover_rate,
    make_price_features,
)


__version__ = "0.10.7"
__author__ = "zengbin93"
__email__ = "zeng_bin8888@163.com"
__date__ = "20251226"


def welcome():
    from czsc import aphorism, envs
    from czsc.utils import get_dir_size, home_path

    print(f"欢迎使用CZSC！当前版本标识为 {__version__}@{__date__}\n")
    aphorism.print_one()

    print(
        f"CZSC环境变量：" f"czsc_min_bi_len = {envs.get_min_bi_len()}; " f"czsc_max_bi_num = {envs.get_max_bi_num()}; "
    )
    if get_dir_size(home_path) > pow(1024, 3):
        print(f"{home_path} 目录缓存超过1GB，请适当清理。调用 czsc.empty_cache_path() 可以直接清空缓存")


if get_dir_size(home_path) > pow(1024, 3):
    print(f"{home_path} 目录缓存超过1GB，请适当清理。调用 czsc.empty_cache_path() 可以直接清空缓存")
